SystemOrganization addCategory: #'SMBase-UI'!
SystemOrganization addCategory: #'SMBase-domain'!
SystemOrganization addCategory: #'SMBase-installer'!
SystemOrganization addCategory: #'SMBase-utilities'!

!VersionNumber methodsFor: '*smbase-printing' stamp: 'gk 1/23/2004 10:13'!
versionString

	^String streamContents: [ :strm | self versionStringOn: strm ]! !

!VersionNumber methodsFor: '*smbase-override' stamp: 'svp 6/18/2002 17:23'!
versionStringOn: strm

	| first |
	first := true.
	numbers do: [ :ea |
		first ifFalse: [strm nextPut: $.].
		first := false.
		ea printOn: strm]
	! !

!ImageSegment methodsFor: '*SMBase-export' stamp: 'gk 11/7/2003 00:21'!
writeForExportOn: fileStream
	"Write the segment on the disk with all info needed to reconstruct it in a new image.  For export.  Out pointers are encoded as normal objects on the disk."

	| temp |
	state = #activeCopy ifFalse: [self error: 'wrong state'].
	temp _ endMarker.
	endMarker _ nil.
	fileStream fileOutClass: nil andObject: self.
		"remember extra structures.  Note class names."
	endMarker _ temp.
! !

!UUID class methodsFor: '*smbase-macsafe' stamp: 'dvf 9/10/2004 23:11'!
fromString36: aString
	"Decode the UUID from a base 36 string using 0-9 and lowercase a-z.
	This is the shortest representation still being able to work as
	filenames etc since it does not depend on case nor characters
	that might cause problems."

	| object num |
	object := self nilUUID.
	num := Integer readFrom: aString asUppercase readStream base: 36.
	16 to: 1 by: -1 do: [:i |
		num size < i
			ifTrue: [object at: i put: 0]
			ifFalse: [object at: i put: (num digitAt: i)]].
	^object! !

!UUID methodsFor: '*smbase-macsafe' stamp: 'gk 1/27/2004 14:15'!
asString36
	"Encode the UUID as a base 36 string using 0-9 and lowercase a-z.
	This is the shortest representation still being able to work as
	filenames etc since it does not depend on case nor characters
	that might cause problems."

	| candidate num |
	num _ 0.
	1 to: self size do: [:i | num _ num + ((256 raisedTo: i - 1) * (self at: i))].
	candidate _ num printStringBase: 36.
	^(candidate copyFrom: 4 to: candidate size) asLowercase! !

!SystemVersion methodsFor: '*smbase-extension' stamp: 'dew 7/9/2004 00:27'!
majorMinorVersion
	"Return the major/minor version number of the form X.Y, without any 'alpha' or 'beta' or other suffix."
	"(SystemVersion new version: 'Squeak3.7alpha') majorMinorVersion" "  -->  'Squeak3.7' "
	"SystemVersion current majorMinorVersion"
	
	| char stream |
	stream := ReadStream on: version, 'x'.
	stream upTo: $..
	char := stream next.
	[char isDigit]
		whileTrue: [char := stream next].
	^ version copyFrom: 1 to: stream position - 1
! !

Object subclass: #RcsDiff
	instanceVariableNames: 'commandLines'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-utilities'!

!RcsDiff class methodsFor: 'instance creation' stamp: 'gh 11/22/2001 23:44'!
lines: aString
	"Create a new RcsDiff file."
	^(self new) commandLines: aString; yourself! !

!RcsDiff methodsFor: 'applying' stamp: 'gh 11/23/2001 11:35'!
applyTo: aString
	"Apply me to given String and return the patched String."

	| space commandStream originalStream nextCommand nextLine lineCount currentLine |
	space _ Character space.
	commandStream _ ReadStream on: commandLines.
	originalStream _ ReadStream on: aString.
	currentLine _ 1.
	^String streamContents: [:stream |
		[nextCommand _ commandStream next.
		nextCommand isNil] whileFalse: [ 
			nextLine _ (commandStream upTo: space) asNumber.
			lineCount _ commandStream nextLine asNumber.
			[currentLine = nextLine]
				whileFalse: [stream nextPutAll: originalStream nextLine; cr. currentLine _ currentLine + 1].
			nextCommand = $d
				ifTrue:[ lineCount timesRepeat: [originalStream nextLine. currentLine _ currentLine + 1]]
				ifFalse:[ nextCommand = $a
							ifTrue:[ stream nextPutAll: originalStream nextLine; cr.
									currentLine _ currentLine + 1.
									lineCount timesRepeat: [
										stream nextPutAll: commandStream nextLine; cr]]]].
		stream nextPutAll: originalStream upToEnd]! !

!RcsDiff methodsFor: 'accessing' stamp: 'gh 11/22/2001 23:44'!
commandLines: aString
	commandLines _ aString! !

Object subclass: #SMFileCache
	instanceVariableNames: 'map'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMFileCache commentStamp: 'gk 3/8/2004 20:10' prior: 0!
A repository for SMSqueakMap downloads. This behaves like a Set, responding to add: and include:, but also package contents may be forcibly refreshed with download:.

The SqueakMap determines what path the cache resides at. Within the cache, there is a 'packages' directory containing UUID-based directories for each package containing further directories for each release. A 'resources' directory stores UUID-based directories for each Resource, with the file stored within that by its original name. Because the cache follows a Set protocol, it can be automatically traversed within Smalltalk's collection protocol, avoiding manual hassles.!

!SMFileCache class methodsFor: 'instance creation' stamp: 'gk 1/23/2004 10:21'!
newFor: aMap
	"This is the default creation method, responsible for ensuring the
	paths and such exist, and filling in defaults."

	^self new forMap: aMap
! !

!SMFileCache methodsFor: 'services' stamp: 'gk 7/10/2004 03:53'!
add: aPackage 
	"Conditionally download the package into the cache.
	Return true on success, otherwise false."

	^(self includes: aPackage)
		ifTrue: [true]
		ifFalse: [self download: aPackage]! !

!SMFileCache methodsFor: 'accessing' stamp: 'gk 1/23/2004 10:26'!
directory
	^map packageCacheDirectory! !

!SMFileCache methodsFor: 'accessing' stamp: 'gk 3/1/2004 10:38'!
directoryForPackage: aPackage
	"Returns the local path for storing the package cache's package file area.
	This also ensures that the path exists."

	| slash path dir |
	slash _ FileDirectory slash.
	path _ 'packages' , slash , aPackage id asString36 , slash.
	dir _ FileDirectory default on: self directory fullName, slash, path.
	dir assureExistence.
	^dir! !

!SMFileCache methodsFor: 'accessing' stamp: 'gk 1/27/2004 14:34'!
directoryForPackageRelease: aPackageRelease
	"Returns the local path for storing the package cache's version of a  
	package file. This also ensures that the path exists."

	| slash path dir |
	slash _ FileDirectory slash.
	path _ 'packages' , slash , aPackageRelease package id asString36 , slash , aPackageRelease automaticVersionString.
	dir _ FileDirectory default on: self directory fullName, slash, path.
	dir assureExistence.
	^dir! !

!SMFileCache methodsFor: 'accessing' stamp: 'gk 1/27/2004 14:34'!
directoryForResource: aResource
	"Returns the local path for storing the package cache's version of a  
	resource file. This also ensures that the path exists."

	| slash path dir |
	slash _ FileDirectory slash.
	path _ 'resources' , slash , aResource id asString36.
	dir _ FileDirectory default on: self directory fullName, slash, path.
	dir assureExistence.
	^dir! !

!SMFileCache methodsFor: 'services' stamp: 'gk 5/23/2004 20:46'!
download: aDownloadable 
	"Download the file for this SM object into the local file cache.
	If the file already exists, delete it.
	No unpacking or installation into the running image."

	| stream file fileName dir |
	[fileName _ aDownloadable downloadFileName.
	fileName
		ifNil: [self inform: 'No download url, can not download.'.
			^ false].
	fileName isEmpty
		ifTrue: [self inform: 'Download url lacks filename, can not download.'.
			^ false].
	dir _ aDownloadable cacheDirectory.
	[(dir fileExists: fileName)
		ifTrue: [dir deleteFileNamed: fileName].
	stream _ aDownloadable downloadUrl asUrl retrieveContents contentStream binary.
	file _ dir newFileNamed: fileName.
	file nextPutAll: stream contents]
		ensure: [file
				ifNotNil: [file close]]]
		on: Error
		do: [^ false].
	^ true! !

!SMFileCache methodsFor: 'initialize' stamp: 'gk 1/23/2004 10:29'!
forMap: aMap
	"Initialize the ache, make sure the cache dir exists."

	map _ aMap! !

!SMFileCache methodsFor: 'services' stamp: 'gk 8/13/2003 15:37'!
includes: anSMObject
	^(anSMObject cacheDirectory)
		fileExists: anSMObject downloadFileName! !

!SMFileCache methodsFor: 'accessing' stamp: 'btr 5/27/2003 16:24'!
map
	^ map! !

Object subclass: #SMInstaller
	instanceVariableNames: 'packageRelease'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-installer'!

!SMInstaller commentStamp: '<historical>' prior: 0!
An installer takes care of installing SqueakMap packages represented by SMCards.
Deciding if a package is installable and instantiating the correct installer class is done on the class side, see implementors of #canInstall:. Two methods need to be implemented by subclasses - download and install. Upgrade can also be specialized by implementing #upgrade, otherwise it will default to #install.!

!SMInstaller class methodsFor: 'changeset utilities' stamp: 'nk 11/30/2002 17:18'!
basicNewChangeSet: newName 
	"This method copied here to ensure SqueakMap is independent of 
	ChangeSorter. "
	Smalltalk
		at: #ChangeSorter
		ifPresentAndInMemory: [:cs | ^ cs basicNewChangeSet: newName].
	(self changeSetNamed: newName)
		ifNotNil: [self error: 'The name ' , newName , ' is already used'].
	^ ChangeSet basicNewNamed: newName! !

!SMInstaller class methodsFor: 'testing' stamp: 'gk 11/16/2003 23:49'!
canInstall: aPackageRelease
	"Nope, I am an abstract class and can not install anything.
	But my subclasses should reimplement this."

	^ false! !

!SMInstaller class methodsFor: 'changeset utilities' stamp: 'gh 10/31/2002 10:11'!
changeSetNamed: newName
	"This method copied here to ensure SqueakMap is independent of ChangeSorter."

	Smalltalk at: #ChangeSorter ifPresentAndInMemory: [ :cs | ^cs changeSetNamed: newName ].
	^ChangeSet allInstances detect: [ :cs | cs name = newName ] ifNone: [ nil ].! !

!SMInstaller class methodsFor: 'instance creation' stamp: 'gk 11/16/2003 23:33'!
classForPackageRelease: aPackageRelease
	"Decide which subclass to instantiate. 
	We detect and return the first subclass
	that wants to handle the release going
	recursively leaf first so that subclasses gets
	first chance if several classes compete over
	the same packages, like for example SMDVSInstaller
	that also uses the .st file extension."

	self subclasses do: [:ea |
		(ea classForPackageRelease: aPackageRelease)
			ifNotNilDo: [:class | ^ class]].
	^(self canInstall: aPackageRelease)
		ifTrue: [self]! !

!SMInstaller class methodsFor: 'deprecated' stamp: 'md 7/16/2004 16:42'!
forPackage: aPackage directory: aDirectory
	"Deprecated. Kept for backwards compatibility. Installing
	a package means installing the latest available release."

	self deprecated: 'Method Deprecated: Use SMInstaller>>forPackageRelease: instead. A directory is not needed in SM2, it has its own cache.'.
	^self forPackageRelease: aPackage lastRelease! !

!SMInstaller class methodsFor: 'deprecated' stamp: 'gk 11/17/2003 00:02'!
forPackageRelease: aPackageRelease
	"Instantiate the first class suitable to install the package release.
	If no installer class is found we raise an Error."

	| class |
	aPackageRelease ifNil: [self error: 'No package release specified to find installer for.'].
	class _ self classForPackageRelease: aPackageRelease.
	^class
		ifNil: [self error: 'No installer found for package ', aPackageRelease name, '.']
		ifNotNil: [class new packageRelease: aPackageRelease]! !

!SMInstaller class methodsFor: 'testing' stamp: 'gk 11/16/2003 23:50'!
isInstallable: aPackageRelease
	"Detect if any subclass can handle the package release."

	aPackageRelease ifNil: [^false].
	^(self classForPackageRelease: aPackageRelease) notNil! !

!SMInstaller class methodsFor: 'testing' stamp: 'gk 11/16/2003 23:49'!
isUpgradeable: aPackageRelease
	"Detect if any subclass can handle the release.
	Currently we assume that upgrade is the same as install."

	^self isInstallable: aPackageRelease! !

!SMInstaller methodsFor: 'services' stamp: 'gk 11/16/2003 21:56'!
download
	"This service should bring the package release to
	the client and also unpack it on disk if needed.
	It will not install it into the running image though.
	Raises errors if operation does not succeed."

	self subclassResponsibility ! !

!SMInstaller methodsFor: 'services' stamp: 'gk 11/16/2003 21:57'!
install
	"This service should bring the package release to the client,
	unpack it if necessary and install it into the image.
	The package release should be notified of the installation using
	'packageRelease noteInstalled'."

	self subclassResponsibility ! !

!SMInstaller methodsFor: 'accessing' stamp: 'gk 11/16/2003 22:03'!
packageRelease: aPackageRelease
	packageRelease _ aPackageRelease! !

!SMInstaller methodsFor: 'private' stamp: 'gk 7/13/2004 02:43'!
silent
	"Can we ask questions?"
	
	^packageRelease ifNotNil: [packageRelease map silent] ifNil: [false]! !

!SMInstaller methodsFor: 'services' stamp: 'gk 7/14/2004 15:38'!
upgrade
	"This service performs an upgrade to the selected release.
	Currently it just defaults to the same operation as an install -
	which is handled fine by Monticello, but not necessarily for
	other formats."

	^self install! !

SMInstaller subclass: #SMSimpleInstaller
	instanceVariableNames: 'fileName dir unpackedFileName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-installer'!

!SMSimpleInstaller commentStamp: '<historical>' prior: 0!
This is a base class that you can subclass if your package format can be downloaded using
a single file url and possibly also be decompressed using gzip.!

SMSimpleInstaller subclass: #SMDefaultInstaller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-installer'!

!SMDefaultInstaller commentStamp: '<historical>' prior: 0!
An installer takes care of installing SqueakMap packages represented by SMCards.
This installer handles packages that consist of classical fileins (single changesets and .st-files) and optional gzip-decompression of those. Deciding if a package is installable and instantiating the correct installer class is done on the class side in SMInstaller, to see how this installer gets chosen - see SMDefaultInstaller class>>canInstall:.

!

SMDefaultInstaller subclass: #SMDVSInstaller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-installer'!

!SMDVSInstaller commentStamp: '<historical>' prior: 0!
This is an installer class for DVS packages. It handles packages categorized with package
format as DVS and with a download filename with extensions .st or .st.gz.

This class can function without DVS installed, needed classes are looked up dynamically.!

!SMDVSInstaller class methodsFor: 'testing' stamp: 'gk 10/1/2003 13:51'!
canInstall: aPackage
	"Can I install this? First we check if class StreamPackageLoader
	is available, otherwise DVS isn't installed.
	Then we check if the package is categorized with package
	format DVS - currently we have hardcoded the id of that category."

	| fileName |
	Smalltalk at: #StreamPackageLoader ifPresentAndInMemory: [ :loader |
		fileName _ aPackage downloadFileName.
		fileName ifNil: [^false].
		fileName _ fileName asLowercase.
		^((fileName endsWith: '.st') or: [fileName endsWith: '.st.gz'])
			and: [aPackage categories includes: "The DVS format category"
					(SMSqueakMap default
						categoryWithId: 'b02f51f4-25b4-4117-9b65-f346215a8e41')]].
	^false! !

!SMDVSInstaller class methodsFor: 'loading' stamp: 'gk 10/1/2003 13:51'!
loadDVS
	"Load the DVS package from SqueakMap."

	SMSqueakMap default installPackageWithId: '100d59d0-bf81-4e74-a4fe-5a2fd0c6b4ec'! !

!SMDVSInstaller methodsFor: 'services' stamp: 'gk 11/16/2003 21:54'!
install
	"Install using DVS."

	| imagePackageLoader streamPackageLoader packageInfo packageManager baseName current new manager |
	self cache; unpack.
	imagePackageLoader _ Smalltalk at: #ImagePackageLoader ifAbsent: [].
	streamPackageLoader _ Smalltalk at: #StreamPackageLoader ifAbsent: [].
	packageInfo _ Smalltalk at: #PackageInfo ifAbsent: [].
	packageManager _ Smalltalk at: #FilePackageManager ifAbsent: [].

	({ imagePackageLoader. streamPackageLoader. packageInfo. packageManager } includes: nil)
		ifTrue: [ (self confirm: ('DVS support is not loaded, but would be helpful in loading ', unpackedFileName, '.
It isn''t necessary, but if you intend to use DVS later it would be a good idea to load it now.
Load it from SqueakMap?'))
			ifTrue: [ self class loadDVS. ^self install ]
			ifFalse: [ ^self fileIn ]].

	baseName _ packageRelease name.
	dir rename: unpackedFileName toBe: (baseName, '.st').
	unpackedFileName _ baseName, '.st'.

	(manager _ packageManager allManagers detect: [ :pm | pm packageName = baseName ] ifNone: [])
		ifNotNil: [
			current _ imagePackageLoader new package: (packageInfo named: baseName).
			new _ streamPackageLoader new stream: (dir readOnlyFileNamed: unpackedFileName).
			(new changesFromBase: current) fileIn ]
		ifNil: [
			self fileIn.
			manager _ packageManager named: baseName. ].

	manager directory: dir.
	packageManager changed: #allManagers.
	packageRelease noteInstalled! !

!SMDefaultInstaller class methodsFor: 'testing' stamp: 'yo 7/5/2004 20:21'!
canInstall: aPackage
	"Answer if this class can install/upgrade the package.
	This installer handles .st, .cs, .st.gz and .cs.gz files."

	| fileName |
	fileName _ aPackage downloadFileName.
	fileName ifNil: [^false].
	fileName _ fileName asLowercase.
	^ FileStream sourceFileSuffixes anySatisfy: [:each | 
		(fileName endsWith: (FileDirectory dot, each)) or: [
			fileName endsWith: (FileDirectory dot, each, '.gz')].
	].
! !

!SMDefaultInstaller methodsFor: 'private' stamp: 'yo 8/17/2004 10:04'!
fileIn
	"Installing in the standard installer is simply filing in.
	Both .st and .cs files will file into a ChangeSet of their own.
	We let the user confirm filing into an existing ChangeSet
	or specify another ChangeSet name if
	the name derived from the filename already exists."
	
	| fileStream |
	((unpackedFileName endsWith: (FileDirectory dot, FileStream st))
		or: [unpackedFileName endsWith: (FileDirectory dot, FileStream cs)])
		ifTrue:[
			fileStream _ dir readOnlyFileNamed: unpackedFileName.
			fileStream setConverterForCode.
			self fileIntoChangeSetNamed: (fileStream localName sansPeriodSuffix)
				fromStream: fileStream.
			^ self].
"	((unpackedFileName endsWith: (FileDirectory dot, FileStream multiSt))
		or: [unpackedFileName endsWith: (FileDirectory dot, FileStream multiCs)])
		ifTrue:[
			fileStream _ dir readOnlyFileNamed: unpackedFileName.
			fileStream converter: UTF8TextConverter new.
			self fileIntoChangeSetNamed: (fileStream localName sansPeriodSuffix)
				fromStream: fileStream.
			^ self].
"
	self error: 'Filename should end with a proper extension'.
! !

!SMDefaultInstaller methodsFor: 'services' stamp: 'gk 11/16/2003 20:52'!
install
	"This service should bring the package to the client,
	unpack it if necessary and install it into the image.
	The package is notified of the installation."

	self cache; unpack; fileIn.
	packageRelease noteInstalled
! !

SMSimpleInstaller subclass: #SMLanguageInstaller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-installer'!

!SMLanguageInstaller class methodsFor: 'testing' stamp: 'mir 7/21/2004 19:28'!
canInstall: aPackage
	"Answer if this class can install the package.
	We handle .translation files optionally compressed."

	| fileName |
	fileName _ aPackage downloadFileName.
	fileName ifNil: [^false].
	fileName _ fileName asLowercase.
	^(fileName endsWith: '.translation') or: [
		(fileName endsWith: '.tra') or: [
			(fileName endsWith: '.tra.gz') or: [
				fileName endsWith: '.translation.gz']]]! !

!SMLanguageInstaller methodsFor: 'services' stamp: 'mir 7/21/2004 19:28'!
install
	"This service should bring the package to the client, 
	unpack it if necessary and install it into the image. 
	The package is notified of the installation."

	self cache; unpack.
	[NaturalLanguageTranslator mergeTranslationFileNamed: unpackedFileName]
			ensure: [packageRelease noteInstalled]! !

SMSimpleInstaller subclass: #SMMcInstaller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-installer'!

!SMMcInstaller commentStamp: 'gk 10/8/2003 14:28' prior: 0!
I am a SMInstaller that knows how to install .mcz (Monticello) files. If Monticello is installed I use that (MCMczReader), otherwise I file in the code more simply using the package MCInstaller (MczInstaller).!

!SMMcInstaller class methodsFor: 'testing' stamp: 'gk 11/14/2003 13:33'!
canInstall: aPackage
	"Is this a Monticello package and do I have MCInstaller
	or Monticello available?"

	| fileName |
	((Smalltalk includesKey: #MCMczReader) or: [
		 Smalltalk includesKey: #MczInstaller])
			ifTrue: [
				fileName _ aPackage downloadFileName.
				fileName ifNil: [^false].
				^ 'mcz' = (FileDirectory extensionFor: fileName) asLowercase].
	^false! !

!SMMcInstaller methodsFor: 'private' stamp: 'ab 8/13/2003 15:17'!
fileIn
	| extension |
	extension _ (FileDirectory extensionFor: fileName) asLowercase.
	extension = 'mcz'
		ifTrue: [self installMcz]
		ifFalse: [self error: 'Cannot install file of type .', extension]! !

!SMMcInstaller methodsFor: 'private' stamp: 'ab 8/8/2003 18:33'!
fullFileName 
	^ dir fullNameFor: fileName! !

!SMMcInstaller methodsFor: 'services' stamp: 'gk 11/16/2003 21:55'!
install
	"This service should bring the package to the client,
	unpack it if necessary and install it into the image.
	The package is notified of the installation."

	self cache; fileIn.
	packageRelease noteInstalled! !

!SMMcInstaller methodsFor: 'private' stamp: 'gk 7/13/2004 02:44'!
installMcz
	"Install the package, we already know that either MCInstaller or Monticello is available."

	| installer monticello |
	installer := MczInstaller.
	(Smalltalk hasClassNamed: #MCMczReader) ifFalse: [
		packageRelease package isInstalled ifTrue: [
			(self silent ifFalse: [
				(self confirm:
'A release of package ''', packageRelease package name, ''' is already installed.
You only have MCInstaller and not Monticello
installed and MCInstaller can not properly upgrade packages.
Do you wish to install Monticello first and then proceed?
If you answer no MCInstaller will be used - but at your own risk.
Cancel cancels the installation.' orCancel: [self error: 'Installation cancelled.'])]
			ifTrue: [false])
				ifTrue: [
					monticello := packageRelease map packageWithName: 'Monticello'.
					monticello lastPublishedRelease
						ifNotNil: [monticello lastPublishedRelease install]
						ifNil: [monticello lastRelease install].
					installer := (Smalltalk at: #MCMczReader)]]
	] ifTrue: [installer := (Smalltalk at: #MCMczReader)].
	installer loadVersionFile: self fullFileName! !

SMSimpleInstaller subclass: #SMProjectInstaller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-installer'!

!SMProjectInstaller commentStamp: '<historical>' prior: 0!
I am a SMInstaller that knows how to install .pr (Project) files.!

!SMProjectInstaller class methodsFor: 'testing' stamp: 'nk 10/26/2002 15:55'!
canInstall: aPackage
	"Answer if this class can install the package.
	We handle .pr files (upper and lowercase)"

	| fileName |
	fileName _ aPackage downloadFileName.
	fileName ifNil: [^false].
	^'pr' = (FileDirectory extensionFor: fileName) asLowercase! !

!SMProjectInstaller methodsFor: 'services' stamp: 'gk 11/16/2003 21:55'!
install
	"This service should bring the package to the client, 
	unpack it if necessary and install it into the image. 
	The package is notified of the installation."

	Project canWeLoadAProjectNow ifFalse: [self error: 'Can not load Project now, probably because not in Morphic.'].
	self cache.
	[[ ProjectLoading openFromDirectory: dir andFileName: fileName ]
		on: ProgressTargetRequestNotification do: [ :ex | ex resume ]]
			ensure: [packageRelease noteInstalled]! !

SMSimpleInstaller subclass: #SMSARInstaller
	instanceVariableNames: 'zip'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-installer'!

!SMSARInstaller commentStamp: '<historical>' prior: 0!
I am a SqueakMap installer that knows how to deal with Zip format change-set archives.
I recognize them by the file extension ".sar" (Squeak Archive).

These have a couple of members with special names:

install/preamble
install/postscript

These are loaded in order. Either or both can further load other members using fileInMemberNamed:.

Inside a postscript or preamble, the pseudo-variable "self" is set to an instance of SARInstaller; you can then get to its ZipArchive using the method "zip". Or you can call its methods for filing in change sets, extracting files, etc.

You can test this loading with:
(SMSARInstaller new) directory: FileDirectory default; fileName: 'test.sar'; fileIn.

See ChangeSet>>fileOutAsZipNamed: for one way to make these files. Here is another way of creating a multi change set archive installable by SqueakMap:

"The following doit will create a .sar file with HVs preamble and postscript as
separate entries and the included changesets included as normal.
Given a preamble as described below this will autoinstall in SqueakMap."
(ChangeSorter changeSetNamed: 'HV')
	fileOutAsZipNamed: 'httpview-021023.sar'
	including: {
		ChangeSorter changeSetNamed: 'HVFixes'.
		ChangeSorter changeSetNamed: 'kom412'}

Preamble in changeset HV that will install the changesets:

"Change Set:		HV
Date:			23 October 2002
Author:			Gran Hultgren

This is my latest developer code drop of HttpView packaged as a Squeak selfextracting archive (courtesy Ned Konz)."

"Standard SqueakMap installing code follows:"
(self isKindOf: SARInstaller) ifTrue:[
	self fileInMemberNamed: 'HVFixes'.
	self fileInMemberNamed: 'kom412'.
	self fileInMemberNamed: 'HV'
]

!

!SMSARInstaller class methodsFor: 'testing' stamp: 'gh 10/31/2002 10:03'!
canInstall: aPackage
	"Answer if this class can install the package.
	We handle it if the filename has the extension
	.sar (upper and lowercase) and SARInstaller is
	present in the image to handle the install."

	| fileName |
	fileName _ aPackage downloadFileName.
	fileName ifNil: [^false].
	Smalltalk at: #SARInstaller ifPresentAndInMemory: [ :installer |
			^'sar' = (FileDirectory extensionFor: fileName) asLowercase].
	^false! !

!SMSARInstaller methodsFor: 'private' stamp: 'gh 10/31/2002 11:19'!
fileIn

	Smalltalk at: #SARInstaller ifPresentAndInMemory: [:installer |
		(installer directory: dir fileName: fileName) fileIn. ^self].
	self error: 'SAR support not installed in image, can not install.'! !

!SMSARInstaller methodsFor: 'services' stamp: 'gk 11/16/2003 21:55'!
install
	"This service should bring the package to the client,
	unpack it if necessary and install it into the image.
	The package is notified of the installation."

	self cache; fileIn.
	packageRelease noteInstalled! !

!SMSimpleInstaller class methodsFor: 'testing' stamp: 'gh 10/22/2002 11:55'!
canInstall: aPackage
	"Answer if this class can install the package.
	This class is abstract so we return false."

	^false! !

!SMSimpleInstaller methodsFor: 'services' stamp: 'gk 11/16/2003 20:52'!
cache
	"Download object into cache if needed.
	Set the directory and fileName for subsequent unpacking and install."

	packageRelease ensureInCache ifTrue: [
		fileName _ packageRelease downloadFileName.
		dir _ packageRelease cacheDirectory]! !

!SMSimpleInstaller methodsFor: 'accessing' stamp: 'gh 10/23/2002 10:54'!
directory
	^dir! !

!SMSimpleInstaller methodsFor: 'services' stamp: 'gk 11/17/2003 00:07'!
download
	"This service downloads the last release of the package
	even if it is in the cache already."

	packageRelease download ifTrue: [
		fileName _ packageRelease downloadFileName.
		dir _ packageRelease cacheDirectory]! !

!SMSimpleInstaller methodsFor: 'services' stamp: 'rbb 3/1/2005 11:12'!
fileIntoChangeSetNamed: aString fromStream: stream
	"We let the user confirm filing into an existing ChangeSet
	or specify another ChangeSet name if
	the name derived from the filename already exists."
	
	| changeSet newName oldChanges global |
	newName := aString.
	changeSet := SMInstaller changeSetNamed: newName.
	changeSet ifNotNil: [
		newName := self silent ifNil: [UIManager default
									request: 'ChangeSet already present, just confirm to overwrite or enter a new name:' 
									initialAnswer: newName]
						ifNotNil: [newName].
		newName isEmpty ifTrue:[self error: 'Cancelled by user'].
		changeSet := SMInstaller changeSetNamed: newName].
		changeSet ifNil:[changeSet := SMInstaller basicNewChangeSet: newName].
		changeSet ifNil:[self error: 'User did not specify a valid ChangeSet name'].
		oldChanges := (SystemVersion current highestUpdate < 5302)
						ifFalse: [global := ChangeSet. ChangeSet current]
						ifTrue: [global := Smalltalk. Smalltalk changes].
 		[global newChanges: changeSet.
		stream fileInAnnouncing: 'Loading ', newName, ' into change set ''', newName, ''''.
		stream close] ensure: [global newChanges: oldChanges]! !

!SMSimpleInstaller methodsFor: 'accessing' stamp: 'gh 10/21/2002 14:39'!
fileName
	^fileName! !

!SMSimpleInstaller methodsFor: 'accessing' stamp: 'gk 8/13/2003 16:07'!
fileName: aFileName
	fileName _ aFileName! !

!SMSimpleInstaller methodsFor: 'accessing' stamp: 'nk 2/22/2004 13:12'!
fullFileName 
	^ self directory fullNameFor: self fileName! !

!SMSimpleInstaller methodsFor: 'services' stamp: 'gk 9/30/2003 16:33'!
unpack
	"This basic installer simply checks the file extension of
	the downloaded file to choose suitable method for unpacking.
	Currently it only supports .gz decompression.
	If a file exists with the same name it is first deleted.
	The unpacked filename is set on succesfull decompression or
	if the file was not recognized as a compressed file."

	| unzipped zipped buffer |
	(fileName endsWith: '.gz')
		ifTrue:[
			unpackedFileName _ fileName copyUpToLast: FileDirectory extensionDelimiter.
			(dir fileExists: unpackedFileName) ifTrue:[ dir deleteFileNamed: unpackedFileName ].
			unzipped _ dir newFileNamed: unpackedFileName.
			unzipped binary.
			zipped _ GZipReadStream on: (dir readOnlyFileNamed: fileName).
			buffer _ ByteArray new: 50000.
			'Extracting ' , fileName
				displayProgressAt: Sensor cursorPoint
				from: 0
				to: zipped sourceStream size
				during: [:bar | 
					[zipped atEnd]
						whileFalse: 
							[bar value: zipped sourceStream position.
							unzipped nextPutAll: (zipped nextInto: buffer)].
					zipped close.
					unzipped close]]
		ifFalse:[unpackedFileName _ fileName]! !

!SMSimpleInstaller methodsFor: 'accessing' stamp: 'gh 10/21/2002 14:39'!
unpackedFileName
	^unpackedFileName! !

Object subclass: #SMObject
	instanceVariableNames: 'id map created updated name summary url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMObject commentStamp: 'gk 9/23/2003 20:26' prior: 0!
SMObject is the abstract superclass for all objects living in an SMSqueakMap.

It has a unique UUID and a reference to the owning SMSqueakMap.
It has timestamps to record the birthtime and the last modification.
It has basic attributes like name, oneline summary and url.





!

SMObject subclass: #SMCategorizableObject
	instanceVariableNames: 'categories resources'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMCategorizableObject commentStamp: 'gk 3/8/2004 19:36' prior: 0!
A categorizable object can be associated with one or more SMCategories.
The relation between the categories and the SMCategorizableObject is bidirectional.
A categorizable object can also have attached resources, SMResource.

The categories are used to classify the categorizable object for different purposes.
Package and package releases are classified in different ways, but so can resources and accounts be.

Accounts can be given various roles using categories.!

!SMCategorizableObject methodsFor: 'private' stamp: 'gh 11/27/2002 12:35'!
addCategory: aCategory
	"Add <aCategory> to me. If I already have it do nothing."

	categories ifNil: [categories _ OrderedCollection new].
	(categories includes: aCategory) ifFalse:[
		aCategory addObject: self.
		categories add: aCategory].
	^aCategory! !

!SMCategorizableObject methodsFor: 'accessing' stamp: 'gk 9/23/2003 20:44'!
categories
	"Lazily initialized."

	^categories ifNil: [OrderedCollection new]! !

!SMCategorizableObject methodsFor: 'initialize-release' stamp: 'gk 9/23/2003 20:44'!
categories: anArray
	"Method used when recreating the object in the image when we need
	to bind the category ids with the actual category objects."

	anArray do: [:i | self addCategory: (map categoryWithId: i)]! !

!SMCategorizableObject methodsFor: 'categories' stamp: 'gk 9/23/2003 20:45'!
categoriesDo: aBlock
	"Evaluate aBlock for each of the categories."

	categories ifNil: [^self].
	categories do: aBlock! !

!SMCategorizableObject methodsFor: 'categories' stamp: 'gh 11/27/2002 12:35'!
categoryForParent: aCategory
	"Answer the one of my categories with parent <aCategory>, if I have it."

	categories ifNil: [^nil].
	^categories detect: [:cat | cat parent = aCategory ] ifNone: [nil]! !

!SMCategorizableObject methodsFor: 'private' stamp: 'gk 8/8/2003 02:35'!
delete
	"Delete me. Disconnect me from my categories."

	super delete.
	self removeFromCategories! !

!SMCategorizableObject methodsFor: 'printing' stamp: 'dew 3/17/2004 16:28'!
describeCategoriesOn: aStream indent: tabs 
	"Show a full listing of categories and their dscription on aStream, indented by the given number of tabs."

	categories isEmptyOrNil
		ifFalse: [aStream cr;
				withAttribute: TextEmphasis bold
				do: [aStream nextPutAll: 'Categories: ']; cr.
			(self categories asSortedCollection: [:a :b | a path < b path])
				do: [:c | 
					aStream tab: tabs.
					c
						parentsDo: [:p | aStream nextPutAll: p name;
								 nextPut: $/].
					aStream nextPutAll: c name;
						 nextPutAll: ' - ';
						
						withAttributes: {TextEmphasis italic. TextIndent tabs: tabs + 1 }
						do: [aStream nextPutAll: c summary];
						 cr]]! !

!SMCategorizableObject methodsFor: 'categories' stamp: 'gh 11/27/2002 12:35'!
hasCategory: aCategory
	"Answer true if I am in it."

	^categories notNil and: [categories includes: aCategory]! !

!SMCategorizableObject methodsFor: 'categories' stamp: 'gk 7/9/2004 02:57'!
hasCategoryOrSubCategoryOf: aCategory
	"Answer true if I am in aCategory or if I am in any
	of its sub categories recursively."

	aCategory allCategoriesDo: [:cat |
		(self hasCategory: cat) ifTrue: [^ true]].
	^false! !

!SMCategorizableObject methodsFor: 'private' stamp: 'gh 11/27/2002 12:35'!
removeCategory: aCategory
	"Remove category from me if I am in it."

	(categories notNil and: [categories includes: aCategory]) ifTrue:[
		aCategory removeObject: self.
		categories remove: aCategory].
	^aCategory! !

!SMCategorizableObject methodsFor: 'private' stamp: 'gh 11/27/2002 12:35'!
removeFromCategories
	"Remove me from all my categories."

	categories ifNotNil:[
		categories copy do: [:cat | self removeCategory: cat ]]! !

SMCategorizableObject subclass: #SMPackageRelease
	instanceVariableNames: 'publisher automaticVersion version note downloadUrl package'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMPackageRelease commentStamp: 'gk 6/26/2003 15:22' prior: 0!
Release-specific package information. This is autonumbered (with a VersionNumber) as well and has a designated version name which can be whatever the maintainer wants.

The instvar publisher holds the name of the SMAccount that owned the package at the time of the release. This is for historic reasons. We don't reference the account itself because those may be deleted.!

!SMPackageRelease class methodsFor: 'instance creation' stamp: 'gk 9/23/2003 20:33'!
newInPackage: aPackage
	"Create a new release for a given package."

	^super new initializeInPackage: aPackage! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gh 11/27/2002 11:22'!
automaticVersion
	"Return the VersionNumber for me."

	^automaticVersion! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gk 8/12/2003 17:21'!
automaticVersionString
	"Return my VersionNumber as a String."

	^automaticVersion versionString! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gk 8/13/2003 15:25'!
cacheDirectory
	^ map cache directoryForPackageRelease: self! !

!SMPackageRelease methodsFor: 'deletion' stamp: 'gh 11/28/2002 21:32'!
delete
	super delete.
	package removeRelease: self! !

!SMPackageRelease methodsFor: 'services' stamp: 'gk 8/13/2003 15:43'!
download
	"Force a download into the cache regardless if it is already there."

	^map cache download: self! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gh 11/27/2002 14:03'!
downloadFileName
	"Cut out the filename from the url."

	downloadUrl isEmpty ifTrue: [^nil].
	^downloadUrl asUrl path last! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gh 11/27/2002 15:31'!
downloadUrl
	^downloadUrl! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gh 11/27/2002 15:31'!
downloadUrl: anObject
	^downloadUrl := anObject! !

!SMPackageRelease methodsFor: 'services' stamp: 'gk 7/13/2004 00:48'!
eitherVersion
	"Return either version:
		1. If the maintainer entered a version then we use that.
		2. Otherwise we use the automatic version with an 'r' prepended."

	^version notEmpty
			ifTrue:[version]
			ifFalse:['r', automaticVersion versionString]! !

!SMPackageRelease methodsFor: 'services' stamp: 'gk 7/10/2004 03:53'!
ensureInCache
	"Makes sure the file is in the cache.
	Return true on success, otherwise false."

	^map cache add: self! !

!SMPackageRelease methodsFor: 'printing' stamp: 'yo 7/26/2004 22:06'!
fullDescription
	"Return a full textual description of the package release."

	| s |
	s := TextStream on: (Text new: 400).
	self describe: self package name withBoldLabel: 'Package name: ' on: s.

	self 
		describe: self version
		withBoldLabel: 'version: '
		on: s.

	categories isEmptyOrNil 
		ifFalse: 
			[s
				cr;
				withAttribute: TextEmphasis bold do: [s nextPutAll: 'Categories: '];
				cr.
			self categoriesDo: 
					[:c | 
					s
						tab;
						withAttribute: TextEmphasis italic
							do: 
								[c parentsDo: 
										[:p | 
										s
											nextPutAll: p name;
											nextPutAll: '/'].
								s nextPutAll: c name];
						nextPutAll: ' - ' , c summary;
						cr].
			s cr].

	self note isEmptyOrNil 
		ifFalse: 
			[s
				cr;
				withAttribute: TextEmphasis bold do: [s nextPutAll: 'Version Comment:'].
			s cr.
			s withAttribute: (TextIndent tabs: 1) do: [s nextPutAll: self note].
			s
				cr;
				cr].
	url isEmptyOrNil 
		ifFalse: 
			[s
				withAttribute: TextEmphasis bold do: [s nextPutAll: 'Homepage:'];
				tab;
				withAttribute: (TextURL new url: url) do: [s nextPutAll: url];
				cr].
	self downloadUrl isEmptyOrNil 
		ifFalse: 
			[s
				withAttribute: TextEmphasis bold do: [s nextPutAll: 'Download:'];
				tab;
				withAttribute: (TextURL new url: self downloadUrl)
					do: [s nextPutAll: self downloadUrl];
				cr].
	^s contents.

! !

!SMPackageRelease methodsFor: 'services' stamp: 'gk 7/12/2004 16:04'!
fullVersion
	"Return version followed by the automatic version
	with r prepended in parenthesis."

	^version, ' (r', automaticVersion versionString, ')'! !

!SMPackageRelease methodsFor: 'view' stamp: 'gk 1/19/2004 12:09'!
getLink: aBuilder

	^aBuilder
		getLinkLocal: '/package/', self package id asString, '/autoversion/',
					self automaticVersion versionString
		text: self packageNameWithVersion! !

!SMPackageRelease methodsFor: 'view' stamp: 'gk 1/19/2004 12:08'!
getShortLink: aBuilder

	^aBuilder
		getLinkLocal: '/package/', self package id asString, '/autoversion/',
					self automaticVersion versionString
		text: self listName! !

!SMPackageRelease methodsFor: 'initialize-release' stamp: 'btr 11/20/2003 00:41'!
initializeInPackage: aPackage
	"Initialize package release.
	Currently we do not support branching so we simply
	pick the next available version number."

	| previous |
	self map: aPackage map id: UUID new.
	package _ aPackage.
	previous _ package lastRelease.
	automaticVersion _ 	previous ifNil: [VersionNumber first]
							ifNotNil: [previous automaticVersion next].
	version _ note _ downloadUrl _ ''! !

!SMPackageRelease methodsFor: 'services' stamp: 'gk 2/16/2004 20:04'!
install
	"Install this package release."

	^map installPackageRelease: self! !

!SMPackageRelease methodsFor: 'testing' stamp: 'gk 8/13/2003 15:32'!
isCached
	"Delegate to last release."

	^map cache includes: self! !

!SMPackageRelease methodsFor: 'testing' stamp: 'dew 7/9/2004 00:18'!
isCompatibleWithCurrentSystemVersion
	"Return true if this release is listed as being compatible with the SystemVersion of the current image.  Only checks major/minor version number; does not differentiate between alpha/beta/gamma releases.  Checks version categories of both the SMPackageRelease and the parent SMPackage."
	^ (self categories, self package categories
		detect:
			[:cat | (cat parent name = 'Squeak versions')
					and: [(SystemVersion new version: cat name) majorMinorVersion = SystemVersion current majorMinorVersion]]
		ifNone: []) notNil
! !

!SMPackageRelease methodsFor: 'testing' stamp: 'gk 10/15/2003 12:37'!
isDownloadable
	"Answer if I can be downloaded.
	We simply verify that the download url
	ends with a filename."

	^self downloadFileName isEmptyOrNil not! !

!SMPackageRelease methodsFor: 'testing' stamp: 'gk 11/16/2003 23:48'!
isInstallable
	"Answer if there is any installer for me.
	This depends typically on the filename of
	the download url, but can in the future
	depend on other things too.
	It does *not* say if the release is installed or not."

	^SMInstaller isInstallable: self! !

!SMPackageRelease methodsFor: 'testing' stamp: 'gk 7/13/2004 13:59'!
isInstalled
	"Answer if this release is installed."

	^(map installedReleaseOf: package) == self! !

!SMPackageRelease methodsFor: 'testing' stamp: 'gh 12/1/2002 19:52'!
isPackageRelease
	^true! !

!SMPackageRelease methodsFor: 'testing' stamp: 'gk 9/23/2003 21:09'!
isPublished
	"It is published when the publisher is set."

	^publisher notNil! !

!SMPackageRelease methodsFor: 'testing' stamp: 'gk 11/17/2003 12:01'!
isUpgradeable
	"Answer if there is any installer that can upgrade me.
	This depends typically on the filename of
	the download url, but can in the future
	depend on other things too.
	It does *not* say if the package is installed or not
	or if there is a newer version available or not."

	^SMInstaller isUpgradeable: self! !

!SMPackageRelease methodsFor: 'printing' stamp: 'gk 7/10/2004 03:35'!
listName
	"Return something suitable for showing in lists.
	We list the manual version after a dash if it is available.
	We don't list the release name."

	^version isEmpty
		ifFalse: [self automaticVersion versionString , '-', version]
		ifTrue: [self automaticVersion versionString] ! !

!SMPackageRelease methodsFor: 'testing' stamp: 'gk 7/13/2004 13:24'!
newerThan: aRelease
	"Answer if this release was made after <aRelease>."
	
	^aRelease automaticVersion < automaticVersion! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gh 11/27/2002 12:43'!
note
	^note! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gh 11/27/2002 12:43'!
note: anObject
	^note := anObject! !

!SMPackageRelease methodsFor: 'services' stamp: 'gk 11/17/2003 01:25'!
noteInstalled
	"This package release was just successfully installed.
	We tell the map so that it can keep track of what
	package releases are installed."

	map noteInstalled: self! !

!SMPackageRelease methodsFor: 'services' stamp: 'gk 1/23/2004 10:01'!
noteUninstalled
	"This package release was just successfully uninstalled.
	We tell the map so that it can keep track of what
	package releases are installed."

	self error: 'Uninstall is not working yet!!'.
	map noteUninstalled: self! !

!SMPackageRelease methodsFor: 'initialize-release' stamp: 'gk 8/8/2003 01:06'!
oldCreated: c updated: u downloadUrl: du maintainer: m registrant: r password: p version: v
	"Deprecated. Only kept for migration from SM 1.0x.
	Method used when recreating from storeOn: format."

	created _ c.
	updated _ u.
	downloadUrl _ du.
	map findOrCreatePublisher: m password: p package: package.
	version _ v! !

!SMPackageRelease methodsFor: 'testing' stamp: 'gk 7/13/2004 13:24'!
olderThan: aRelease
	"Answer if this release was made before <aRelease>."
	
	^automaticVersion < aRelease automaticVersion! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gh 3/15/2003 20:55'!
package
	"Get the package that I belong to."

	^package! !

!SMPackageRelease methodsFor: 'private' stamp: 'gh 11/27/2002 11:02'!
package: aPackage
	"Set when I am created."

	package _ aPackage! !

!SMPackageRelease methodsFor: 'printing' stamp: 'gk 7/10/2004 03:38'!
packageNameWithVersion
	"Return '<packageName> <autoVersion>-<version>' like:
		'SqueakMap 5-0.92'	"

	^package name, ' ', self listName! !

!SMPackageRelease methodsFor: 'services' stamp: 'gh 11/27/2002 11:04'!
previousRelease
	"Return the release before me.
	Returns nil if there is none."

	^package previousReleaseFor: self! !

!SMPackageRelease methodsFor: 'printing' stamp: 'gk 7/10/2004 03:38'!
printName
	"Return a String identifying object without context."

	^self packageNameWithVersion! !

!SMPackageRelease methodsFor: 'printing' stamp: 'gk 12/9/2003 00:13'!
printOn: aStream

	aStream nextPutAll: self class name, '[', self packageNameWithVersion, ']'! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gk 6/26/2003 14:59'!
publisher
	^publisher! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gk 6/26/2003 15:00'!
publisher: anObject
	publisher := anObject! !

!SMPackageRelease methodsFor: 'services' stamp: 'gk 7/13/2004 00:49'!
smartVersion
	"This method is used to ensure that we always have a
	version name for the package release even if the maintainer didn't
	bother to enter one. Is is calculated like this:
		1. If the maintainer entered a version then we use that.
		2. Otherwise we use the automatic version with an 'r' prepended.
		3. If the release is not published we enclose it in parenthesis."

	^ self isPublished ifTrue: [self eitherVersion] ifFalse: ['(', self eitherVersion, ')']! !

!SMPackageRelease methodsFor: 'printing' stamp: 'gk 11/14/2003 00:11'!
type

	^'Package release'! !

!SMPackageRelease methodsFor: 'services' stamp: 'gk 7/14/2004 17:42'!
upgrade
	"Upgrade this package release if there is a new release available."

	| newRelease |
	newRelease _ package lastPublishedReleaseForCurrentSystemVersionNewerThan: self.
	newRelease ifNotNil: [(SMInstaller forPackageRelease: newRelease) upgrade]! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gh 11/27/2002 15:30'!
version
	^version! !

!SMPackageRelease methodsFor: 'accessing' stamp: 'gh 11/27/2002 15:30'!
version: anObject
	^version := anObject! !

!SMPackageRelease methodsFor: 'view' stamp: 'gh 3/15/2003 19:37'!
viewFor: uiObject
	"This is a double dispatch mechanism for multiple views
	for multiple uis."

	^uiObject packageReleaseViewOn: self! !

SMCategorizableObject subclass: #SMRootedObject
	instanceVariableNames: 'homeMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

SMRootedObject subclass: #SMAccount
	instanceVariableNames: 'initials email signature password newPassword advogatoId objects coObjects isAdmin'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMAccount commentStamp: 'gk 3/8/2004 19:29' prior: 0!
SMAccount is the object representing a user account in SqueakMap.

It keeps track of the email address, developer initials and two passwords used to access the account. There is also an advogatoId (people.squeakfoundation.org username) and a signature field (not used yet). The flag isAdmin is a crude way of marking a user as a superuser, this will possibly be changed in the future and instead expressed using a category.

Passwords are stored as secure hashes. The extra password (newPassword) is used when the regular password is forgotten - it is then randomly set and an email is sent out containing it to the registered email. This enables the original password to still work. When logging in, the user gets a chance to enter a new regular password overwriting the old one and clearing the random new password in the process.

The instvar objects holds all SMPersonalObjects that this account "owns" - these are typically instances of SMPackages and SMResources, but are not limited to be.

The instvar coObjects holds all SMPersonalObjects that this account is co-maintaining - these are typically instances of SMPackages and SMResources.

Finally the account also maintains a directory with uploaded files on the server. This directory has the UUID of the account as its name and it located under sm/accounts!

!SMAccount methodsFor: 'objects' stamp: 'gk 11/11/2003 16:16'!
addCoObject: anObject
	"Add <anObject> to this account.
	Only called from #addMaintainer:."
	
	(coObjects includes: anObject)
		ifFalse:[coObjects add: anObject]! !

!SMAccount methodsFor: 'objects' stamp: 'gk 11/11/2003 20:49'!
addObject: anObject
	"Add <anObject> to this account. Also makes sure the
	reverse reference is correct."
	
	(objects includes: anObject) ifFalse:[
		objects add: anObject.
		anObject owner: self.
		map addObject: anObject]! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 11/13/2003 23:06'!
advogatoId
	^advogatoId! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 11/13/2003 23:05'!
advogatoId: aString
	advogatoId _ aString! !

!SMAccount methodsFor: 'objects' stamp: 'gk 11/11/2003 17:37'!
coPackageWithId: anIdString
	"Return the correct package or nil."

	| uuid |
	uuid _ UUID fromString: anIdString.
	^self coPackages detect: [:p | p id = uuid ] ifNone: [nil]! !

!SMAccount methodsFor: 'objects' stamp: 'gk 11/11/2003 16:13'!
coPackages
	"Return all co-maintained packages."

	^coObjects select: [:o | o isPackage].! !

!SMAccount methodsFor: 'passwords' stamp: 'gk 8/4/2003 14:50'!
correctPassword: aPassword
	"We store the password as a SHA hash so that we can let the slave maps
	have them too. Also check the optional new random password."

	| try |
	aPassword isEmptyOrNil ifTrue:[^false].
	try _ SecureHashAlgorithm new hashMessage: aPassword.
	^password = try or: [newPassword = try]! !

!SMAccount methodsFor: 'passwords' stamp: 'gk 11/17/2003 10:22'!
createRandomPassword

	| randomPass |
	randomPass _ String streamContents: [:stream | 10 timesRepeat: [ stream nextPut: 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789' atRandom]].
	self setNewPassword: randomPass.
	^randomPass! !

!SMAccount methodsFor: 'files' stamp: 'gk 3/8/2004 19:26'!
delete
	"Delete this account. First delete all SM objects we own
	and disconnect this account from those we co-maintain."

	objects do: [:o | o delete].
	coObjects do: [:co | co removeMaintainer: self].
	super delete
! !

!SMAccount methodsFor: 'files' stamp: 'gk 3/8/2004 19:26'!
deleteFiles: fileNames
	"Delete all fileNames from the uploads directory."

	| dir |
	dir _ self uploadsDirectory.
	fileNames do: [:fn | dir deleteFileNamed: fn]
! !

!SMAccount methodsFor: 'files' stamp: 'gk 8/14/2003 14:39'!
directory
	"Get the directory for the account."

	| dir |
	dir _ (map directory directoryNamed: 'accounts') assureExistence; yourself.
	^(dir directoryNamed: id asString) assureExistence; yourself
! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 6/26/2003 14:26'!
email
	^email! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 6/26/2003 14:27'!
email: address
	email _ address! !

!SMAccount methodsFor: 'files' stamp: 'gk 3/8/2004 19:26'!
entries
	"Return all file entries in the upload directory."

	^self uploadsDirectory entries! !

!SMAccount methodsFor: 'files' stamp: 'gk 8/15/2003 12:08'!
files
	"Return filenames for uploaded files."

	^self uploadsDirectory fileNames
! !

!SMAccount methodsFor: 'view' stamp: 'gk 11/12/2003 14:14'!
getLink: aBuilder
	"Return a link for using on the web."

	^aBuilder getLinkLocal: '/accountbyid/', id asString text: self nameWithInitials! !

!SMAccount methodsFor: 'initialize-release' stamp: 'gk 11/17/2003 11:49'!
initialize
	"Initialize account."

	super initialize.
	initials _ signature _ advogatoId _ ''.
	isAdmin _ false.
	objects _ OrderedCollection new.
	coObjects _ OrderedCollection new! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 7/30/2003 14:10'!
initials
	^initials! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 8/8/2003 19:11'!
initials: aString
	"If these are changed we need to update the dictionary in the map."

	initials ~= aString ifTrue: [
		initials _ aString.
		map clearUsernames]! !

!SMAccount methodsFor: 'testing' stamp: 'gk 6/26/2003 16:47'!
isAccount
	^true! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 11/17/2003 11:49'!
isAdmin
	^isAdmin ifNil: [false] ifNotNil: [isAdmin]! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 11/17/2003 11:47'!
isAdmin: aBoolean
	isAdmin _ aBoolean! !

!SMAccount methodsFor: 'view' stamp: 'gk 8/4/2003 14:10'!
logout
	"Automatically called upon logout. Do nothing."! !

!SMAccount methodsFor: 'objects' stamp: 'gk 8/7/2003 21:00'!
moveObject: aPersonalObject toAccount: anAccount
	"Transfer the ownership of the given personal object to <anAccount>."

	self removeObject: aPersonalObject.
	anAccount addObject: aPersonalObject! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 7/9/2004 20:06'!
nameAndEmail
	^name , ' <', email, '>'! !

!SMAccount methodsFor: 'view' stamp: 'gk 8/5/2003 13:16'!
nameWithInitials
	"Return name and developer initials within parentheses."

	^name, ' (', (initials isEmptyOrNil ifTrue: ['not entered'] ifFalse: [initials]) , ')'! !

!SMAccount methodsFor: 'files' stamp: 'gk 9/23/2003 21:16'!
newFile: fileName block: aBlock
	"Create a new file. Let <aBlock> fill the file with content by calling it with a stream."

	| dir stream |
	dir _ self uploadsDirectory.
	[(dir fileExists: fileName) ifTrue:[dir deleteFileNamed: fileName].
	stream _ dir newFileNamed: fileName.
	aBlock value: stream] ensure: [stream close]! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 8/4/2003 16:34'!
newPassword
	"Get the parallell password hash."

	^newPassword! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 8/4/2003 15:16'!
newPassword: aHashNumber
	"Set the parallell password hash."

	newPassword _ aHashNumber! !

!SMAccount methodsFor: 'testing' stamp: 'gk 11/11/2003 17:24'!
owns: anObject
	^objects includes: anObject! !

!SMAccount methodsFor: 'objects' stamp: 'gk 8/7/2003 02:50'!
packageWithId: anIdString
	"Return the correct package or nil."

	| uuid |
	uuid _ UUID fromString: anIdString.
	^self packages detect: [:p | p id = uuid ] ifNone: [nil]! !

!SMAccount methodsFor: 'objects' stamp: 'gk 8/4/2003 13:58'!
packages
	"Return all owned packages."

	^objects select: [:o | o isPackage].! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 8/4/2003 15:42'!
password
	"Get the password hash."

	^password! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 8/4/2003 11:51'!
password: aHashNumber
	"Set the password hash."

	password _ aHashNumber! !

!SMAccount methodsFor: 'view' stamp: 'gk 8/8/2003 00:09'!
publicViewFor: uiObject
	"This is a double dispatch mechanism for multiple views
	for multiple uis."

	^uiObject publicAccountViewOn: self! !

!SMAccount methodsFor: 'objects' stamp: 'gk 11/11/2003 16:16'!
removeCoObject: anObject
	"Remove <anObject> from this account.
	Only called from #removeMaintainer:."

	(coObjects includes: anObject) ifTrue: [
		coObjects remove: anObject]! !

!SMAccount methodsFor: 'objects' stamp: 'gk 8/7/2003 20:56'!
removeObject: anObject
	"Remove <anObject> from this account. Also makes sure the
	reverse reference is cleared."

	(objects includes: anObject) ifTrue: [
		anObject owner: nil.
		objects remove: anObject]! !

!SMAccount methodsFor: 'passwords' stamp: 'gk 6/26/2003 14:02'!
setNewPassword: aString
	"Set a new parallell password the user can use to get in
	if the old password is forgotten. We don't delete the old
	password since the request for this new password is made
	anonymously. Note that the password is stored as a secured
	hash large integer."

	newPassword _ SecureHashAlgorithm new hashMessage: aString! !

!SMAccount methodsFor: 'passwords' stamp: 'gk 6/26/2003 13:58'!
setPassword: aString
	"We also clear the random extra password."

	password _ SecureHashAlgorithm new hashMessage: aString.
	newPassword _ nil! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 8/4/2003 15:56'!
signature
	"Get the signature."

	^signature! !

!SMAccount methodsFor: 'accessing' stamp: 'gk 8/4/2003 15:57'!
signature: aSignature
	"Set the signature."

	signature _ aSignature! !

!SMAccount methodsFor: 'files' stamp: 'gk 8/14/2003 14:28'!
streamForFile: fileName
	"Return a readonly stream for file <fileName>.
	If the file does not exist return nil."

	| stream |
	[stream _ self uploadsDirectory oldFileNamed: fileName]
		on: FileDoesNotExistException do: [^nil].
	^stream! !

!SMAccount methodsFor: 'printing' stamp: 'gk 11/14/2003 00:12'!
type

	^'Account'! !

!SMAccount methodsFor: 'files' stamp: 'gk 8/14/2003 14:23'!
uploadsDirectory
	"Get the directory for uploaded files, create it if missing."

	^(self directory directoryNamed: 'uploads') assureExistence; yourself
! !

!SMAccount methodsFor: 'view' stamp: 'gk 6/26/2003 16:01'!
viewFor: uiObject
	"This is a double dispatch mechanism for multiple views
	for multiple uis."

	^uiObject accountViewOn: self! !

SMRootedObject subclass: #SMPersonalObject
	instanceVariableNames: 'owner maintainers rss'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMPersonalObject commentStamp: 'gk 9/23/2003 21:28' prior: 0!
SMPersonalObject is the abstract base class for things that belong/are owned by a user account in SqueakMap. Most things are personal objects - but the SMCategories aren't for example.

A personal object has a reference to the SMAccount owning it. It also has a potential list of maintainers - other accounts that also can modify the object.!

SMPersonalObject subclass: #SMDocument
	instanceVariableNames: 'description author'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMDocument commentStamp: 'gk 3/8/2004 19:56' prior: 0!
An SMDocument refers typically to a piece of digital information accessible through a URL. :-)
This means it can be downloaded and cached.

The instvar description describes the document and instvar author references the name and/or email of the original author.

SMDocument has two subclasses - SMPackage and SMResource. Since SqueakMap is primarily meant for
keeping track of installable source packages of Squeak software, a specific subclass handles those.

The main difference is that an SMPackage has SMPackageReleases - versions of the package. An SMResource has no versions and is always referred to as it is *now*, but it does have a field for keeping track of the current version name.!

!SMDocument methodsFor: 'accessing' stamp: 'gk 3/8/2004 19:51'!
author
	^author! !

!SMDocument methodsFor: 'accessing' stamp: 'gk 3/8/2004 19:52'!
author: aString
	author _ aString! !

!SMDocument methodsFor: 'accessing' stamp: 'gk 3/8/2004 19:52'!
description
	^description! !

!SMDocument methodsFor: 'accessing' stamp: 'gk 3/8/2004 19:52'!
description: aString
	description _ aString! !

!SMDocument methodsFor: 'cache' stamp: 'gk 3/8/2004 19:51'!
ensureInCache
	"Makes sure the file is in the cache."

	self subclassResponsibility ! !

!SMDocument methodsFor: 'initialize-release' stamp: 'gk 3/8/2004 19:53'!
initialize

	description := author := ''! !

!SMDocument methodsFor: 'cache' stamp: 'gk 8/12/2003 17:09'!
isCached
	"Is the file corresponding to me in the local file cache?"

	self subclassResponsibility ! !

SMDocument subclass: #SMPackage
	instanceVariableNames: 'releases packageInfoName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMPackage commentStamp: 'gh 11/22/2002 14:42' prior: 0!
An SMCard is a library card describing a Squeak code package. It is the object representing a particular
package and currently it has information about the current version available of the package.

It contains a minimal amount of information as follows:

name			A String with a name for the package. The card has an id too so the name is not what makes the card unique.
currentVersion	A String with the readable version of the package. This can follow whatever style the author wants.
summary		A oneline String describing shortly what the package is in one sentence.
description		A 5-10 lines String describing the package in full.
url				A String with the url to the WWW homepage of the package if it has one.
downloadUrl		A String with the url to a download of the package.
author			A String with the original author in format 'Joe Schmoe <joe@schmoe.com>'.
maintainer		A String with the current package maintainer in format 'Joe Schmoe <joe@schmoe.com>'.
registrator		A String with the the person registrating the card in format 'Joe Schmoe <joe@schmoe.com>'.
password		A String with the password that the registrator selected when registering the card.
versionComment	A String describing the specific version of the package.
categories		An OrderedCollection with the SMCategories that this card belongs to.

Of all these variables only the last one - categories is not a simple String. When being stored or transmitted we use the UUIDs of the SMCategory objects instead and then "reconnect" them using real references.!

!SMPackage methodsFor: 'private' stamp: 'gk 10/15/2003 12:46'!
addRelease: aRelease
	"Add the release."

	releases add: aRelease.
	aRelease package: self! !

!SMPackage methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:11'!
author
	^author! !

!SMPackage methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:24'!
author: aString
	author _ aString! !

!SMPackage methodsFor: 'cache' stamp: 'gk 8/12/2003 17:20'!
cacheDirectory
	^ self lastRelease cacheDirectory! !

!SMPackage methodsFor: 'deprecated' stamp: 'btr 11/20/2003 00:23'!
created: c updated: u name: n currentVersion: v summary: s description: d url: ur downloadUrl: du author: a maintainer: m registrator: r password: p categories: cats 
	"Deprecated. Only kept for migration from SM 1.0x.
	Method used when recreating from storeOn: format.
	A few attributes are moved over from the card and the release
	is given the same categories as the card to begin with."

	self isReleased ifFalse:[self newRelease].
	self lastRelease
		oldCreated: c updated: u downloadUrl: du maintainer: m registrant: r password: p version: v;
		categories: cats.
	self categories: cats.
	created _ c.
	updated _ u.
	name _ n.
	summary _ s.
	description _ d.
	url _ ur.
	author _ a	! !

!SMPackage methodsFor: 'accessing' stamp: 'gk 7/12/2004 16:05'!
currentVersion
	^self isPublished ifTrue: [self lastPublishedRelease version]! !

!SMPackage methodsFor: 'private' stamp: 'gh 11/28/2002 21:25'!
delete
	"Delete me. Delete my releases."

	super delete.
	self deleteReleases! !

!SMPackage methodsFor: 'private' stamp: 'gk 8/8/2003 10:01'!
deleteReleases
	"Delete my releases."

	releases do: [:release | release delete]! !

!SMPackage methodsFor: 'accessing' stamp: 'gh 3/19/2002 20:11'!
description
	^description! !

!SMPackage methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:23'!
description: aString
	description _ aString! !

!SMPackage methodsFor: 'cache' stamp: 'gk 3/8/2004 19:56'!
download
	"Force download into cache."

	self isReleased ifFalse: [self error: 'There is no release for this package to download.'].
	^self lastRelease download! !

!SMPackage methodsFor: 'services' stamp: 'btr 11/20/2003 00:30'!
downloadFileName
	"Cut out the filename from the url."
	self isReleased ifFalse: [self error: 'There is no release for this package to download.'].
	^self lastRelease downloadFileName! !

!SMPackage methodsFor: 'accessing' stamp: 'btr 11/20/2003 00:30'!
downloadUrl
	self isReleased ifFalse: [self error: 'There is no release for this package to download.'].
	^self lastRelease downloadUrl! !

!SMPackage methodsFor: 'deprecated' stamp: 'gh 11/27/2002 14:04'!
downloadUrl: aString
	"Deprecated."
	self halt.! !

!SMPackage methodsFor: 'cache' stamp: 'btr 11/20/2003 00:30'!
ensureInCache
	"Makes sure the file is in the cache."
	self isReleased ifFalse: [self error: 'There is no release for this package to download.'].
	^self lastRelease ensureInCache ! !

!SMPackage methodsFor: 'accessing' stamp: 'yo 7/28/2004 17:14'!
fullDescription
	"Return a full textual description of the package. 
	Most of the description is taken from the last release."
	| s publishedRelease |
	s := TextStream on: (Text new: 400).

	self
		describe: name
		withBoldLabel: 'Name:		'
		on: s.

	summary isEmptyOrNil
		ifFalse: [self
				describe: summary
				withBoldLabel: 'Summary:	'
				on: s ].

	author isEmptyOrNil
		ifFalse: [s
				withAttribute: TextEmphasis bold
				do: [s nextPutAll: 'Author:'];
				 tab;
				 tab.
			s
				withAttribute: (PluggableTextAttribute
						evalBlock: [self userInterface
										sendMailTo: (SMUtilities stripEmailFrom: author)
										regardingPackageRelease: self lastRelease])
				do: [s nextPutAll: author];
				 cr].
	self owner
		ifNotNil: [s
				withAttribute: TextEmphasis bold
				do: [s nextPutAll: 'Owner:'];
				 tab; tab.
			s
				withAttribute: (PluggableTextAttribute
						evalBlock: [self userInterface
										sendMailTo: self owner email
										regardingPackageRelease: self lastRelease])
				do: [s nextPutAll: self owner nameAndEmail];	
				 cr].

	self maintainers isEmpty ifFalse: [
		s withAttribute: TextEmphasis bold do: [s nextPutAll: 'Co-maintainers:']; tab.
		self maintainers do: [:com |
			com = self maintainers first ifFalse: [s nextPutAll: ', '].
			s
				withAttribute:
					(PluggableTextAttribute
						evalBlock: [self userInterface
									sendMailTo: com email
									regardingPackageRelease: self lastRelease])
				do: [s nextPutAll: com nameAndEmail]].
				s cr].

	description isEmptyOrNil
		ifFalse: [s cr.
			s
				withAttribute: TextEmphasis bold
				do: [s nextPutAll: 'Description:'].
			s cr.
			s
				withAttribute: (TextIndent tabs: 1)
				do: [s next: (description findLast: [ :c | c isSeparator not ]) putAll: description].
			s cr ].

	self describeCategoriesOn: s indent: 1.

	s cr.
	publishedRelease _ self lastPublishedRelease.
	self
		describe: (self publishedVersion ifNil: ['<not published>'])
		withBoldLabel: 'Published version: '
		on: s.
	self isPublished ifTrue: [
		s
			withAttribute: TextEmphasis bold do: [ s nextPutAll: 'Created: ' ];
			print: publishedRelease created;
			cr.
			self note isEmptyOrNil
				ifFalse: [s
					withAttribute: TextEmphasis bold
					do: [s nextPutAll: 'Release note:'].
			s cr.
			s
				withAttribute: (TextIndent tabs: 1)
				do: [s nextPutAll: publishedRelease note].
			s cr ]].

	url isEmptyOrNil
		ifFalse: [s cr;
				withAttribute: TextEmphasis bold
				do: [s nextPutAll: 'Homepage:'];
				 tab;
				withAttribute: (TextURL new url: url)
				do: [s nextPutAll: url];
				 cr].

	^ s contents! !

!SMPackage methodsFor: 'view' stamp: 'gh 7/30/2002 16:35'!
getLink: aBuilder
	"Return a link for using on the web."

	^aBuilder getLinkLocal: '/package/', id asString text: name! !

!SMPackage methodsFor: 'initialize-release' stamp: 'gk 3/8/2004 19:52'!
initialize
	"Initialize package."

	super initialize.
	releases := OrderedCollection new! !

!SMPackage methodsFor: 'installation' stamp: 'gk 7/14/2004 17:32'!
install
	"Install the latest newer published version for this version of Squeak."

	^map installPackage: self! !

!SMPackage methodsFor: 'installation' stamp: 'gk 11/17/2003 02:27'!
installedRelease
	"Return the installed release.
	We ask the map. Return nil if this package is not installed."

	^map installedReleaseOf: self! !

!SMPackage methodsFor: 'installation' stamp: 'gk 11/17/2003 10:14'!
installedVersion
	"Return the version String for the installed version.
	We ask the map. Return nil if this package is not installed."

	^self installedRelease ifNotNilDo: [:r | r smartVersion]! !

!SMPackage methodsFor: 'testing' stamp: 'gk 7/13/2004 15:11'!
isAvailable
	"Answer if I am old or not installed regardless of
	if there is installer support for me. It also does
	not care if the newer release is not published
	or no for this Squeak version."

	^self isOld or: [self isInstalled not]! !

!SMPackage methodsFor: 'testing' stamp: 'gk 7/13/2004 14:55'!
isCached
	"Is the last release corresponding to me in the local file cache?
	NOTE: This doesn't honour #published nor if the release is
	intended for the current Squeak version."

	^self isReleased and: [self lastRelease isCached]! !

!SMPackage methodsFor: 'testing' stamp: 'btr 11/20/2003 00:35'!
isDownloadable
	"Answer if I can be downloaded."

	^self isReleased and: [self lastRelease isDownloadable]! !

!SMPackage methodsFor: 'testing' stamp: 'gk 7/15/2004 17:21'!
isInstallable
	"Answer if any of my releases can be installed."

	^ releases anySatisfy: [:rel | rel isInstallable]! !

!SMPackage methodsFor: 'testing' stamp: 'gh 10/25/2002 11:08'!
isInstallableAndNotInstalled
	"Answer if there is any installer that
	can install me and I am not yet installed."

	^self isInstallable and: [self isInstalled not]! !

!SMPackage methodsFor: 'testing' stamp: 'gk 11/17/2003 11:02'!
isInstalled
	"Answer if any version of me is installed."

	^(map installedReleaseOf: self) notNil! !

!SMPackage methodsFor: 'testing' stamp: 'gk 7/13/2004 15:10'!
isOld
	"Answer if I am installed and there also is a
	newer version available *regardless* if it is
	not published or not for this Squeak version.
	This is for people who want to experiment!!"

	| installed |
	installed := map installedReleaseOf: self.
	^installed
		ifNil: [false]
		ifNotNil: [
			self releases anySatisfy: [:r |
				r newerThan: installed ]]! !

!SMPackage methodsFor: 'testing' stamp: 'gh 12/1/2002 19:40'!
isPackage
	^true! !

!SMPackage methodsFor: 'testing' stamp: 'gk 9/23/2003 21:35'!
isPublished
	"Answer if I have public releases."

	^releases anySatisfy: [:rel | rel isPublished]! !

!SMPackage methodsFor: 'testing' stamp: 'btr 11/20/2003 00:05'!
isReleased
	^ releases isEmpty not! !

!SMPackage methodsFor: 'testing' stamp: 'gk 7/13/2004 15:37'!
isSafeToInstall
	"Answer if I am NOT installed and there also is a
	published version for this version of Squeak available."

	^self isInstalled not and: [
		self lastPublishedReleaseForCurrentSystemVersion notNil]! !

!SMPackage methodsFor: 'testing' stamp: 'gk 7/14/2004 16:15'!
isSafelyAvailable
	"Answer if I am old or not installed regardless of
	if there is installer support for me. The
	newer release should be published
	and meant for this Squeak version."

	^self isSafeToInstall or: [self isSafelyOld]! !

!SMPackage methodsFor: 'testing' stamp: 'gk 7/14/2004 17:16'!
isSafelyOld
	"Answer if I am installed and there also is a
	newer published version for this version of Squeak available."

	| installed |
	installed _ self installedRelease.
	^installed ifNil: [false] ifNotNil: [
		^(self lastPublishedReleaseForCurrentSystemVersionNewerThan: installed) notNil]! !

!SMPackage methodsFor: 'testing' stamp: 'gk 7/14/2004 17:16'!
isSafelyOldAndUpgradeable
	"Answer if I am installed and there also is a
	newer published version for this version of Squeak available
	that can be upgraded to (installer support)."

	| installed newRelease |
	installed _ self installedRelease.
	^installed ifNil: [false] ifNotNil: [
		newRelease _ self lastPublishedReleaseForCurrentSystemVersionNewerThan: installed.
		^newRelease ifNil: [false] ifNotNil: [newRelease isUpgradeable]]! !

!SMPackage methodsFor: 'testing-delegated' stamp: 'btr 11/20/2003 00:37'!
isUpgradeable
	"Answer if there is any installer that can upgrade me."

	^self isReleased and: [self lastRelease isUpgradeable]! !

!SMPackage methodsFor: 'testing' stamp: 'gh 10/25/2002 10:47'!
isUpgradeableAndOld
	"Answer if there is any installer that
	can upgrade me and I can be upgraded."

	^self isUpgradeable and: [self isOld]! !

!SMPackage methodsFor: 'testing' stamp: 'gh 10/25/2002 11:23'!
isUpgradeableAndOldOrInstallableAndNotInstalled
	"Well, duh. Isn't it obvious? :-)
	Is the package available now for automatic install or automatic upgrade?"

	^self isUpgradeableAndOld and: [self isInstallableAndNotInstalled]! !

!SMPackage methodsFor: 'services' stamp: 'gk 2/16/2004 20:14'!
lastPublishedRelease
	"Return the latest published release."

	^releases isEmpty ifTrue: [nil] ifFalse: [
		releases reversed detect: [:r | r isPublished] ifNone:[nil]]! !

!SMPackage methodsFor: 'services' stamp: 'gk 7/13/2004 13:28'!
lastPublishedReleaseForCurrentSystemVersion
	"Return the latest published release marked
	as compatible with the current SystemVersion."

	^releases isEmpty ifTrue: [nil] ifFalse: [
		releases reversed detect: [:r |
			r isPublished and: [r isCompatibleWithCurrentSystemVersion]]
				ifNone:[nil]]! !

!SMPackage methodsFor: 'services' stamp: 'gk 7/14/2004 17:15'!
lastPublishedReleaseForCurrentSystemVersionNewerThan: aRelease
	"Return the latest published release marked
	as compatible with the current SystemVersion
	that is newer than the given release."

	^releases isEmpty ifTrue: [nil] ifFalse: [
		releases reversed detect: [:r |
			(r isPublished and: [r newerThan: aRelease])
				and: [r isCompatibleWithCurrentSystemVersion]]
				 	ifNone:[nil]]! !

!SMPackage methodsFor: 'services' stamp: 'gk 8/4/2003 11:49'!
lastRelease
	"Return the latest release."

	^releases isEmpty ifTrue: [nil] ifFalse: [releases last]! !

!SMPackage methodsFor: 'accessing' stamp: 'gk 1/25/2004 16:23'!
maintainer
	"Deprecated"

	^self owner! !

!SMPackage methodsFor: 'deprecated' stamp: 'btr 11/20/2003 00:38'!
modulePath: p moduleVersion: v moduleTag: t versionComment: vc
	"Deprecated. Only kept for migration from SM 1.0x.
	Method used when recreating from storeOn: format."

	self isReleased ifTrue: [self lastRelease note: vc]! !

!SMPackage methodsFor: 'installation' stamp: 'gk 11/18/2003 02:14'!
nameWithVersionLabel
	^name, ' (', self versionLabel, ')'! !

!SMPackage methodsFor: 'private' stamp: 'gk 8/7/2003 21:28'!
newRelease
	"Create a new release."

	^releases addLast: (map newObject: (SMPackageRelease newInPackage: self))! !

!SMPackage methodsFor: 'accessing' stamp: 'gk 7/12/2004 17:15'!
note
	^self isPublished ifTrue: [self lastPublishedRelease note]! !

!SMPackage methodsFor: 'accessing' stamp: 'gk 11/6/2003 15:24'!
packageInfoName
	^packageInfoName! !

!SMPackage methodsFor: 'accessing' stamp: 'gk 11/6/2003 15:24'!
packageInfoName: aString
	packageInfoName _ aString! !

!SMPackage methodsFor: 'services' stamp: 'gh 11/27/2002 12:33'!
previousReleaseFor: aPackageRelease
	"If there is none, return nil."
	
	^releases before: aPackageRelease ifAbsent: [nil]! !

!SMPackage methodsFor: 'accessing' stamp: 'gk 7/12/2004 17:13'!
publishedVersion
	^self isPublished ifTrue: [self lastPublishedRelease version]! !

!SMPackage methodsFor: 'services' stamp: 'gk 11/18/2003 17:39'!
releaseWithAutomaticVersion: aVersion
	"Look up a specific package release of mine. Return nil if missing.
	They are few so we just do a #select:."

	^releases detect: [:rel | rel automaticVersion = aVersion ] ifNone: [nil]! !

!SMPackage methodsFor: 'services' stamp: 'gk 11/18/2003 17:39'!
releaseWithAutomaticVersionString: aVersionString
	"Look up a specific package release of mine. Return nil if missing.
	They are few so we just do a #select:."

	^self releaseWithAutomaticVersion: aVersionString asVersion! !

!SMPackage methodsFor: 'services' stamp: 'gh 3/15/2003 18:50'!
releaseWithId: anIdString 
	"Look up a specific package release of mine. Return nil if missing.
	They are few so we just do a #select:."

	| anId |
	anId _ UUID fromString: anIdString.
	releases detect: [:rel | rel id = anId ].
	^nil! !

!SMPackage methodsFor: 'services' stamp: 'gk 11/18/2003 17:41'!
releaseWithVersion: aVersionString
	"Look up a specific package release of mine. Return nil if missing.
	They are few so we just do a #select:."

	^releases detect: [:rel | rel version = aVersionString ] ifNone: [nil]! !

!SMPackage methodsFor: 'accessing' stamp: 'gh 12/1/2002 21:21'!
releases
	^releases! !

!SMPackage methodsFor: 'private' stamp: 'gh 11/28/2002 21:33'!
removeRelease: aRelease
	"Remove the release."

	releases remove: aRelease! !

!SMPackage methodsFor: 'services' stamp: 'gk 7/13/2004 14:37'!
smartVersion
	"Delegate to last published release for this SystemVersion."

	| r |
	r := self lastPublishedReleaseForCurrentSystemVersion.
	^r ifNotNil: [r smartVersion] ifNil: ['']! !

!SMPackage methodsFor: 'printing' stamp: 'gk 11/14/2003 00:12'!
type

	^'Package'! !

!SMPackage methodsFor: 'installation' stamp: 'gk 7/14/2004 17:31'!
upgrade
	"Upgrade to the latest newer published version for this version of Squeak."

	| installed |
	installed _ self installedRelease.
	installed
		ifNil: [self error: 'No release installed, can not upgrade.']
		ifNotNil: [^installed upgrade]! !

!SMPackage methodsFor: 'installation' stamp: 'gk 7/14/2004 17:43'!
upgradeOrInstall
	"Upgrade to or install the latest newer published version for this version of Squeak."

	| installed |
	installed _ self installedRelease.
	installed
		ifNil: [^self install]
		ifNotNil: [^installed upgrade]! !

!SMPackage methodsFor: 'installation' stamp: 'gk 7/14/2004 17:16'!
versionLabel
	"Return a label indicating installed and available version as:
		'1.0'      = 1.0 is installed and no new published version for this version of Squeak is available
		'1.0->1.1' = 1.0 is installed and 1.1 is published for this version of Squeak
		'->1.1'    = No version is installed and 1.1 is published for this version of Squeak
		'->(1.1)	 = No version is installed and there is only a non published version available for this version of Squeak

	The version showed is the one that #smartVersion returns.
	If a version name is in parenthesis it is not published."

	| installedVersion r r2 |
	r _ self installedRelease.
	r ifNotNil: [
		installedVersion _ r smartVersion.
		r2 _ self lastPublishedReleaseForCurrentSystemVersionNewerThan: r]
	ifNil: [
		installedVersion _ ''.
		r2 _ self lastPublishedReleaseForCurrentSystemVersion ].
	^r2 ifNil: [installedVersion ] ifNotNil: [installedVersion, '->', r2 smartVersion].! !

!SMPackage methodsFor: 'view' stamp: 'gh 3/15/2003 18:05'!
viewFor: uiObject
	"This is a double dispatch mechanism for multiple views
	for multiple uis."

	^uiObject packageViewOn: self! !

SMDocument subclass: #SMResource
	instanceVariableNames: 'subjectId version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMResource commentStamp: 'gk 9/23/2003 21:54' prior: 0!
A resource is a document that is NOT a package. Thus, it is used for all the things interesting to register on SM that aren't packages. The are three major differences with resources:

- A resource keeps no track of version history like packages do with package releases. It only has a field for the current version.
- A resource can be embedded inside the map instead of being a document reached by a URL.
- A resource can be associated with another SMObject, the subject.

However, resources respond to some of the same actions as PackageReleases.!

SMResource subclass: #SMEmbeddedResource
	instanceVariableNames: 'content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMEmbeddedResource commentStamp: 'gk 3/8/2004 20:01' prior: 0!
An embedded resource is simply a String resource, held in instvar #content, that is stored inside the map. No download is needed.
This means that embedded resources should be "small" and typically only be used for resources that
are needed to be available at all times without downloading. A typical example is meta data for other SMObjects.

!

!SMEmbeddedResource methodsFor: 'accessing' stamp: 'gk 3/8/2004 19:58'!
content
	^content! !

!SMEmbeddedResource methodsFor: 'accessing' stamp: 'btr 5/28/2003 02:05'!
content: aString
	content _ aString! !

!SMEmbeddedResource methodsFor: 'accessing' stamp: 'gk 3/8/2004 19:58'!
contents
	"Answers the content string."

	^content! !

!SMEmbeddedResource methodsFor: 'services' stamp: 'gk 3/8/2004 19:58'!
download
	"A dummy method to respond as other resources would."

	^true! !

!SMEmbeddedResource methodsFor: 'services' stamp: 'gk 8/13/2003 15:54'!
ensureInCache
	"Makes sure the file is in the cache."

	^true! !

!SMEmbeddedResource methodsFor: 'testing' stamp: 'gk 8/12/2003 17:10'!
isCached
	"Is the file corresponding to me in the local file cache?
	Well consider it as true since I am embedded in the map."

	^true! !

SMResource subclass: #SMExternalResource
	instanceVariableNames: 'downloadUrl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMExternalResource commentStamp: 'gk 3/8/2004 20:09' prior: 0!
An external resource is a downloadable resource.
The instance variable downloadUrl holds the URL to the resource and the resource is cacheable in the FileCache for the SqueakMap.
An external resource can be used for any kind of document that is to be attached to another SMObject.!

!SMExternalResource methodsFor: 'accessing' stamp: 'btr 5/28/2003 04:13'!
cacheDirectory
	^ map cache directoryForResource: self! !

!SMExternalResource methodsFor: 'accessing' stamp: 'gk 3/8/2004 20:08'!
contents
	"Return the contents of a stream from the downloaded resource.
	Not yet tested, this resource returns the stream and not its contents."

	map cache add: self.
	^self cacheDirectory readOnlyFileNamed: self downloadFileName! !

!SMExternalResource methodsFor: 'services' stamp: 'gk 8/13/2003 15:42'!
download
	"Force a download into the cache regardless if it is already there."

	^map cache download: self! !

!SMExternalResource methodsFor: 'accessing' stamp: 'btr 5/28/2003 04:13'!
downloadFileName
	"Cut out the filename from the url."

	downloadUrl isEmpty ifTrue: [^nil].
	^downloadUrl asUrl path last! !

!SMExternalResource methodsFor: 'accessing' stamp: 'btr 5/28/2003 04:14'!
downloadUrl
	^ downloadUrl! !

!SMExternalResource methodsFor: 'accessing' stamp: 'btr 5/28/2003 04:15'!
downloadUrl: anUrl
	downloadUrl _ anUrl! !

!SMExternalResource methodsFor: 'services' stamp: 'gk 8/13/2003 15:54'!
ensureInCache
	"Makes sure the file is in the cache."

	^map cache add: self! !

!SMExternalResource methodsFor: 'testing' stamp: 'gk 8/13/2003 15:32'!
isCached
	"Is the file corresponding to me in the local file cache?"

	^map cache includes: self! !

!SMResource class methodsFor: 'instance creation' stamp: 'btr 5/28/2003 04:31'!
forString: aString
	^ SMEmbeddedResource new content: aString! !

!SMResource class methodsFor: 'instance creation' stamp: 'btr 5/28/2003 04:30'!
forUrl: anUrl
	^ SMExternalResource new downloadUrl: anUrl; yourself! !

!SMResource methodsFor: 'testing' stamp: 'btr 5/28/2003 04:25'!
isResource
	^ true! !

!SMResource methodsFor: 'accessing' stamp: 'gk 9/23/2003 21:55'!
version
	^ version! !

!SMResource methodsFor: 'accessing' stamp: 'gk 9/23/2003 21:55'!
version: aVersion
	version _ aVersion! !

!SMPersonalObject methodsFor: 'maintainers' stamp: 'gk 11/11/2003 20:16'!
addMaintainer: anAccount
	"Add anAccount as a maintainer."

	maintainers ifNil: [maintainers _ OrderedCollection new].
	maintainers add: anAccount.
	anAccount addCoObject: self! !

!SMPersonalObject methodsFor: 'deletion' stamp: 'gk 8/8/2003 02:27'!
delete
	"Disconnect from owner and maintainers."

	super delete.
	owner removeObject: self.
	maintainers ifNotNil: [
		maintainers copy do: [:m | self removeMaintainer: m]]! !

!SMPersonalObject methodsFor: 'maintainers' stamp: 'avi 1/29/2004 02:48'!
isOwnerOrMaintainer: anAccount
	^ owner = anAccount or: [self maintainers includes: anAccount]! !

!SMPersonalObject methodsFor: 'accessing' stamp: 'gk 8/7/2003 20:57'!
maintainers
	"Return all maintainers."

	^maintainers ifNil: [#()]! !

!SMPersonalObject methodsFor: 'accessing' stamp: 'gk 8/7/2003 20:56'!
owner
	^owner! !

!SMPersonalObject methodsFor: 'accessing' stamp: 'gk 8/7/2003 20:56'!
owner: anAccount
	owner _ anAccount! !

!SMPersonalObject methodsFor: 'maintainers' stamp: 'gk 11/11/2003 20:16'!
removeMaintainer: anAccount
	"Remove anAccount as a maintainer."

	maintainers ifNil: [^self].
	maintainers remove: anAccount.
	anAccount removeCoObject: self! !

!SMPersonalObject methodsFor: 'accessing' stamp: 'gk 11/6/2003 15:23'!
rss
	^rss! !

!SMPersonalObject methodsFor: 'accessing' stamp: 'gk 11/6/2003 15:22'!
rss: anUrl
	rss _ anUrl! !

SMObject subclass: #SMCategory
	instanceVariableNames: 'mandatory subCategories parent objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMCategory commentStamp: 'gk 3/8/2004 19:44' prior: 0!
An SMCategory is a "tag" that can be attached to SMCategorizableObjects in order to classify them.

The SMCategories are arranged in a strict hierarchy and each SMCategory both knows it's parent and it's subcategories.
The instvar objects holds all SMObjects belonging to this category.
Instvars name and summary are already inherited from SMObject and describe the category.
The instvar url can be used to refer to a web page that can explain the category in more detail, typically a page at the Squeak Swiki.
SMCategory adds an instance variable called mandatory holding a Set with the classes (SMPackage, SMPackageRelease, SMAccount, SMResource etc) that must belong to at least one subcategory of this SMCategory. Obviously not many categories will be mandatory for each class.

The category tree is maintained by a few trusted people so that chaos will not reign. :-)
!

!SMCategory methodsFor: 'private' stamp: 'gh 6/28/2002 12:46'!
addCategory: cat
	"Add a category as a subcategory to self.
	The collection of subcategories is lazily instantiated."

	subCategories ifNil: [subCategories _ OrderedCollection new].
	cat parent ifNotNil: [cat parent removeCategory: cat ].
	subCategories add: cat.
	cat parent: self.
	^cat! !

!SMCategory methodsFor: 'deprecated' stamp: 'gk 11/17/2003 13:52'!
addMandatoryClass: aClass
	"Add <aClass> as one of the SMObject types that I am mandatory for."

	mandatory ifNil: [mandatory _ Set new].
	mandatory add: aClass! !

!SMCategory methodsFor: 'private' stamp: 'gk 9/23/2003 21:59'!
addObject: anObject
	"Add <anObject> to this category. This should only be called
	from SMCategorizableObject>>addCategory: to ensure consistency."
	
	(objects includes: anObject) ifFalse:[objects add: anObject]! !

!SMCategory methodsFor: 'services' stamp: 'gk 11/17/2003 10:48'!
allCategoriesDo: aBlock
	"Evaluate <aBlock> for all categories below me including me,
	bottom up breadth-first."

	self allSubCategoriesDo: aBlock.
	aBlock value: self! !

!SMCategory methodsFor: 'services' stamp: 'gk 7/9/2004 02:59'!
allSubCategoriesDo: aBlock
	"Evaluate <aBlock> for all categories below me NOT including me,
	bottom up breadth-first."

	subCategories ifNil: [^self].
	subCategories do: [:sub |
		sub allSubCategoriesDo: aBlock.
		aBlock value: sub]! !

!SMCategory methodsFor: 'services' stamp: 'gh 8/5/2002 17:10'!
categoryBefore
	"Return the category listed before me in my parent.
	If I am first or I have no parent, return nil."

	parent isNil ifTrue:[^nil].
	parent subCategories first = self ifTrue:[^nil].
	^parent subCategories before: self
	! !

!SMCategory methodsFor: 'deprecated' stamp: 'gk 11/17/2003 13:51'!
created: c updated: u name: n summary: s url: uu mandatory: m parentId: anId
	"Method used when recreating from storeOn: format.
	Note: That #addCategory: will set the parent variable."

	created _ c.
	updated _ u.
	name _ n.
	summary _ s.
	url _ uu.
	m ifTrue: [self addMandatoryClass: SMPackage].
	anId ifNotNil: [(map categoryWithId: anId) addCategory: self]! !

!SMCategory methodsFor: 'private' stamp: 'gk 8/8/2003 10:03'!
delete
	"Delete me. Disconnect me from my objects and my parent.
	Then delete my subcategories."

	super delete.
	self removeFromObjects; removeFromParent.
	self subCategories do: [:c | c delete ]
	
	! !

!SMCategory methodsFor: 'view' stamp: 'gk 11/17/2003 14:50'!
getLink: aView
	"Return a link for using on the web."

	^aView linklocal: '/category/', id asString text: name! !

!SMCategory methodsFor: 'testing' stamp: 'gh 12/1/2002 20:24'!
hasSubCategories
	^subCategories isEmptyOrNil not! !

!SMCategory methodsFor: 'testing' stamp: 'gk 8/7/2003 22:31'!
includes: anObject
	"Answer if <anObject> is in this category."

	^objects includes: anObject! !

!SMCategory methodsFor: 'initialize-release' stamp: 'gk 11/17/2003 13:23'!
initialize
	super initialize.
	name _ summary _ url _ ''.
	objects _ OrderedCollection new! !

!SMCategory methodsFor: 'testing' stamp: 'gh 12/1/2002 19:51'!
isCategory
	^true! !

!SMCategory methodsFor: 'testing' stamp: 'gh 6/27/2002 13:31'!
isTopCategory
	^parent isNil! !

!SMCategory methodsFor: 'accessing' stamp: 'gk 11/17/2003 13:51'!
mandatory
	^mandatory! !

!SMCategory methodsFor: 'accessing' stamp: 'gk 11/17/2003 14:01'!
mandatory: aSet
	mandatory _ aSet! !

!SMCategory methodsFor: 'accessing' stamp: 'gk 11/17/2003 21:35'!
mandatoryFor: aClass
	"Is this category mandatory for instances of <aClass>?"

	^mandatory ifNil: [false] ifNotNil: [mandatory includes: aClass]! !

!SMCategory methodsFor: 'services' stamp: 'gh 8/5/2002 14:36'!
move: cat toAfter: before
	"Move a category to be after the category <before>."

	subCategories remove: cat.
	before ifNil: [subCategories addFirst: cat] ifNotNil: [subCategories add: cat after: before]! !

!SMCategory methodsFor: 'accessing' stamp: 'gh 8/1/2002 16:54'!
objects
	"Return all objects in this category."

	^objects! !

!SMCategory methodsFor: 'accessing' stamp: 'gk 8/7/2003 23:42'!
packages
	"Return all packages in this category."

	^objects select: [:p | p isPackage]! !

!SMCategory methodsFor: 'accessing' stamp: 'gh 6/27/2002 16:01'!
parent
	^parent! !

!SMCategory methodsFor: 'private' stamp: 'gh 6/28/2002 12:41'!
parent: aCategory
	"Change the parent category.
	This method relies on that somebody else
	updates the parent's subCategories collection."
	
	parent _ aCategory! !

!SMCategory methodsFor: 'deprecated' stamp: 'gh 12/2/2002 20:00'!
parentId: anId
	"Method used when recreating from storeOn: format.
	Note: That #addCategory: will set the parent variable."

	(map categoryWithId: anId) addCategory: self! !

!SMCategory methodsFor: 'services' stamp: 'gh 8/1/2002 17:30'!
parentsDo: aBlock
	"Run a block for all my parents starting from the top."

	parent ifNotNil: [
		parent parentsDo: aBlock.
		aBlock value: parent]! !

!SMCategory methodsFor: 'printing' stamp: 'gk 1/29/2004 00:14'!
path
	"Return my name with a full path of my
	parent names separated with slashes like:
		'Squeak versions/Squeak3.5' "

	^String streamContents: [:s |
		self parentsDo: [:cat |
			s nextPutAll: cat name; nextPutAll: '/'].
		s nextPutAll: self name]! !

!SMCategory methodsFor: 'printing' stamp: 'gh 8/16/2002 06:04'!
printOn: aStream

	aStream nextPutAll: self class name, ': ', name! !

!SMCategory methodsFor: 'services' stamp: 'gh 6/27/2002 16:20'!
removeCategory: cat
	"Remove a category from subcategories of self.
	No error handling is done here."

	cat parent: nil.
	^subCategories remove: cat! !

!SMCategory methodsFor: 'private' stamp: 'gk 11/14/2003 11:53'!
removeDeepFromObjects
	"Remove myself from my objects and then ask
	my subCategories to do the same."

	objects copy do: [:obj | obj removeCategory: self].
	subCategories do: [:cat | cat removeDeepFromObjects]! !

!SMCategory methodsFor: 'private' stamp: 'gh 12/1/2002 20:03'!
removeFromObjects
	"Remove myself from my objects."

	objects copy do: [:obj | obj removeCategory: self]! !

!SMCategory methodsFor: 'private' stamp: 'gh 12/1/2002 20:28'!
removeFromParent
	"Remove me from my parent."

	parent ifNotNil: [parent removeCategory: self]! !

!SMCategory methodsFor: 'services' stamp: 'gk 3/8/2004 19:48'!
removeMandatoryClass: aClass
	"Remove <aClass> as one of the SMObject types that I am mandatory for."

	mandatory ifNotNil: [mandatory remove: aClass ifAbsent: [^nil]]! !

!SMCategory methodsFor: 'private' stamp: 'gh 12/1/2002 20:28'!
removeObject: anObject
	"Remove <anObject> from this category. This should only be called
	from SMCategorizableObject>>removeCategory: to ensure consistency."
	
	^objects remove: anObject! !

!SMCategory methodsFor: 'accessing' stamp: 'gh 6/27/2002 12:25'!
subCategories
	subCategories ifNil: [^#()].
	^subCategories! !

!SMCategory methodsFor: 'printing' stamp: 'gk 11/14/2003 00:13'!
type

	^'Category'! !

!SMCategory methodsFor: 'view' stamp: 'gh 8/1/2002 16:38'!
viewFor: uiObject
	"This is a double dispatch mechanism for multiple views
	for multiple uis."

	^uiObject categoryViewOn: self! !

!SMObject class methodsFor: 'instance creation' stamp: 'nk 7/30/2004 10:57'!
newIn: aMap
	"Create a new object in a given map with an UUID to ensure unique identity."

	^(self new) map: aMap id: UUID new! !

!SMObject class methodsFor: 'instance creation' stamp: 'nk 7/30/2004 10:56'!
newIn: aMap withId: anUUIDString
	"Create a new object in a given SMSqueakMap with a given UUID as a String.
	This method is used when we create instances from a logfile etc."

	^(self new) map: aMap id: (UUID fromString: anUUIDString)! !

!SMObject methodsFor: 'transactions' stamp: 'gk 6/6/2003 23:34'!
changesFrom: originalObject on: stream
	"Store messagesends that express the difference compared to
	the given original on the stream."

	stream crtab; nextPutAll: 'updated:'.
	updated storeOn: stream.
	originalObject name = name ifFalse:[
		stream nextPut: $;; crtab; nextPutAll: 'name: '.
		name storeOn: stream].
	originalObject summary = summary ifFalse:[
		stream nextPut: $;; crtab; nextPutAll: 'summary: '.
		summary storeOn: stream].
	originalObject url = url ifFalse:[
		stream nextPut: $;; crtab; nextPutAll: 'url: '.
		url storeOn: stream]! !

!SMObject methodsFor: 'transactions' stamp: 'gk 9/26/2003 00:05'!
commit
	"Write a snapshot of the object to disk."

! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/16/2002 23:12'!
created
	^TimeStamp fromSeconds: created! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/16/2002 23:12'!
created: c
	created _ c! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/16/2002 23:12'!
createdAsSeconds
	^created! !

!SMObject methodsFor: 'transactions' stamp: 'gh 12/1/2002 21:30'!
definition
	"Return a String with a definition used for logging."

	self subclassResponsibility ! !

!SMObject methodsFor: 'transactions' stamp: 'gh 8/15/2002 08:38'!
definition: isUpdate
	"Return a String with a definition used for logging."

	self subclassResponsibility ! !

!SMObject methodsFor: 'transactions' stamp: 'gh 12/2/2002 19:58'!
definitionOn: stream
	"Stream out messages to create this object."

	stream crtab; nextPutAll: 'created: '.
	created storeOn: stream.
	stream nextPut: $;; crtab; nextPutAll: 'updated:'.
	updated storeOn: stream.
	stream nextPut: $;; crtab; nextPutAll: 'name: '.
	name storeOn: stream.	
	stream nextPut: $;; crtab; nextPutAll: 'summary: '.
	summary storeOn: stream.
	stream nextPut: $;; crtab; nextPutAll: 'url: '.
	url storeOn: stream! !

!SMObject methodsFor: 'deletion' stamp: 'gk 8/8/2003 10:10'!
delete
	"Delete from map."

	map deleteObject: self! !

!SMObject methodsFor: 'printing' stamp: 'gk 11/14/2003 14:22'!
describe: string withBoldLabel: label on: stream
	"Helper method for doing styled text."

	stream withAttribute: (TextEmphasis bold) do: [ stream nextPutAll: label ].
	stream nextPutAll: string; cr! !

!SMObject methodsFor: 'transactions' stamp: 'gk 9/26/2003 00:46'!
endXMLOn: stream
	"Stream out endTag for this object."

	stream nextPutAll: '</', self startTag, '>'! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/16/2002 23:12'!
id
	^id! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:11'!
id: anId
	id _ anId! !

!SMObject methodsFor: 'initialize-release' stamp: 'gk 9/23/2003 21:02'!
initialize
	"Initialize the receiver."

	updated _ created _ TimeStamp current asSeconds.
	name _ summary _ url _ ''.! !

!SMObject methodsFor: 'testing' stamp: 'gk 6/26/2003 16:47'!
isAccount
	^false! !

!SMObject methodsFor: 'testing' stamp: 'gh 12/1/2002 19:51'!
isCategory
	^false! !

!SMObject methodsFor: 'testing' stamp: 'gh 12/1/2002 19:40'!
isPackage
	^false! !

!SMObject methodsFor: 'testing' stamp: 'gh 12/1/2002 19:51'!
isPackageRelease
	^false! !

!SMObject methodsFor: 'testing' stamp: 'gh 12/1/2002 19:51'!
isResource
	^false! !

!SMObject methodsFor: 'transactions' stamp: 'gh 12/1/2002 20:25'!
logDeleteOn: aStream
	"Log a deletion of me on the stream."

	aStream cr; nextChunkPut: 'self deleteObjectWithId: ', id asString storeString! !

!SMObject methodsFor: 'logging' stamp: 'gh 8/15/2002 08:39'!
logOn: aStream
	"Log me on the stream."

	aStream cr; nextChunkPut: (self definition: false)! !

!SMObject methodsFor: 'accessing' stamp: 'gh 8/15/2002 08:50'!
map
	^map! !

!SMObject methodsFor: 'accessing' stamp: 'gk 11/6/2003 14:46'!
map: aMap
	map _ aMap! !

!SMObject methodsFor: 'initialize-release' stamp: 'gk 9/23/2003 21:02'!
map: aMap id: anId
	"Initialize the receiver."

	self initialize.
	map _ aMap.
	id _ anId! !

!SMObject methodsFor: 'accessing' stamp: 'gh 6/26/2002 15:31'!
name
	^name! !

!SMObject methodsFor: 'accessing' stamp: 'gh 6/26/2002 15:31'!
name: aName
	name _ aName! !

!SMObject methodsFor: 'printing' stamp: 'gk 7/10/2004 03:39'!
printName
	"Return a String identifying receiver without a context.
	Default is name."

	^self name! !

!SMObject methodsFor: 'printing' stamp: 'gk 8/4/2003 11:56'!
printOn: aStream

	aStream nextPutAll: self class name, '[', name, ']'! !

!SMObject methodsFor: 'mail' stamp: 'gh 10/22/2002 16:49'!
randomPhrase
	"Pick a nice phrase."

	^#('Debug safely' 'Happy Squeaking' 'Just do it' 'Yours truly' 'Stay a Squeaker' 'Squeak rocks') atRandom! !

!SMObject methodsFor: 'updating' stamp: 'gk 9/24/2003 22:40'!
stampAsUpdated
	"This method should be called whenever the object is modified."

	updated _ TimeStamp current asSeconds! !

!SMObject methodsFor: 'transactions' stamp: 'gk 9/26/2003 00:46'!
startXMLOn: stream
	"Stream out startTag to create this object."

	stream nextPutAll: '<?xml version="1.0" encoding="UTF-8"?>';cr.
	stream nextPutAll: '<', self startTag, ' created="'.
	created storeOn: stream.
	stream nextPutAll: '" updated="'.
	updated storeOn: stream.
	stream nextPutAll: '" name="'.
	name storeOn: stream.
	stream nextPutAll: '" summary="'.
	summary storeOn: stream.
	stream nextPutAll: '" url="'.
	url storeOn: stream.
	stream nextPutAll: '">';cr! !

!SMObject methodsFor: 'accessing' stamp: 'gh 6/26/2002 15:33'!
summary
	^summary! !

!SMObject methodsFor: 'accessing' stamp: 'gh 6/26/2002 15:33'!
summary: aString
	summary _ aString! !

!SMObject methodsFor: 'printing' stamp: 'gk 11/14/2003 00:11'!
type

	^'Object'! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:44'!
updated
	^TimeStamp fromSeconds: updated! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:44'!
updated: c
	updated _ c! !

!SMObject methodsFor: 'accessing' stamp: 'gh 3/17/2002 01:45'!
updatedAsSeconds
	^updated! !

!SMObject methodsFor: 'accessing' stamp: 'gh 6/26/2002 15:36'!
url
	^url! !

!SMObject methodsFor: 'accessing' stamp: 'gh 6/26/2002 15:36'!
url: aString
	url _ aString! !

!SMObject methodsFor: 'accessing' stamp: 'gh 11/27/2002 12:21'!
userInterface
	"Return the object that we use for interacting with the user."

	^SMUtilities! !

Object subclass: #SMSqueakMap
	instanceVariableNames: 'packages accounts objects categories dir transactionCounter firstTransactionNumber daysBacklog adminPassword installedPackages installCounter fileCache users mutex dirtyList isDirty checkpointNumber silent'
	classVariableNames: 'DefaultMap ServerList'
	poolDictionaries: ''
	category: 'SMBase-domain'!

!SMSqueakMap commentStamp: 'gk 8/3/2003 22:39' prior: 0!
A SqueakMap is a Squeak code package metacatalog - a catalog of all available Squeak packages. SMSqueakMap is the class for the domain model.

One master instance lives on a server on the Internet at map1.squeakfoundation.org (but there are fallback servers too). Then each Squeak connected to the Internet has one instance that is synchronized incrementally against the master. This way every user can have an updated catalog of all available Squeak software on the planet locally on their machine. :-)

Typically you only need one instance of SMSqueakMap per image and it is held by a singleton class variable reached with "SMSqueakMap default". If it is not there it will then be created together with it's own filelog on disk by default in the "sm" directory created in your default directory.

The map is continously mirrored in a log-file and every operation on the map open and close the log-file to ensure nothing is lost. Synching with the master is the only action that affects the map so you can actually use the same map from multiple images (they will by default use the same logfile if the images have the same default directory) and multiple tools (SMLoader, SMBrowser or others) as long as you always do a "SMSqueakMap default reloadLog" before anything else when you switch image!!

An instance of SMSqueakMap contains instances of SMCard which are "library cards" for Squeak packages and SMCategories which are centrally registered values that can be included as attributes in an SMCard. An example of such a category would be different kinds of licenses, topic etc. etc.

It also contains instances of SMRepository which are registered Squeak module repositories and their respective top module. This way the SqueakMap can resolve a module path to the correct repository. Note: this is for 3.3a Modules which is not really used yet.

Finally SMSqueakMap also contains "in image only" state (not logged in logfile) about what packages and versions of them have been installed in the image. This is the Dictionary installedPackages.
--------------------
"Simplest use of SMSqueakMap - this will create a map if you don't have one and open a simple UI""
SMLoader open

Use these doits to play "hands on" with a SqueakMap.

"Creating another SqueakMap in directory 'slavemap' instead of default 'sm'"
Smalltalk at: #AnotherSqueakMap put: (SMSqueakMap newIn: 'slavemap')

"Update the default map by synching it with the master - check Transcript for some numbers."
SMSqueakMap default loadUpdates

"If the map is broken in some way, reload it from the log"
SMSqueakMap default reloadLog

"If you want to rotate the log and save the current SqueakMap in the new log"
SMSqueakMap default saveNewLog
!

!SMSqueakMap class methodsFor: 'changelog replay' stamp: 'rbb 2/18/2005 13:26'!
askUser
	"Ask user about how to handle a replayed installation note
	when there is no current SqueakMap in the image."

	| choice |
	[choice _ UIManager default chooseFrom: #('Yes' 'No' 'More info')
				title:
'There is no SqueakMap in this image,
do you wish to create/recreate it? (typical answer is Yes)' .
			choice = 3] whileTrue: [self inform:
'When packages are installed using SqueakMap a little mark is made
in the change log. When an image is reconstructed from the changelog
these marks are intended to keep your map informed about what packages
are installed. You probably already have a map on disk which will automatically be
reloaded if you choose ''Yes'', otherwise an new empty map will be created.
If you choose ''No'', it will only result in that SqueakMap will not know that this package
is installed in your image.
If you are still unsure - answer ''Yes'' since that is probably the best.'].
	^choice = 1! !

!SMSqueakMap class methodsFor: 'bootstrap upgrade' stamp: 'gk 1/17/2004 13:03'!
bootStrap
	"Bootstrap upgrade. Only used when SqueakMap itself is too old to
	communicate with the server. This relies on the existence of a package
	called SqueakMap that is a .st loadscript. The loadscript needs to do its
	own changeset management."

	| server url |
	server _ self findServer.
	server ifNotNil: ["Ok, found a SqueakMap server"
		url _ (('http://', server, '/sm/packagebyname/squeakmap/downloadurl')
				asUrl retrieveContents content) asUrl.
		(url retrieveContents content unzipped readStream)
				fileInAnnouncing: 'Upgrading SqueakMap...']! !

!SMSqueakMap class methodsFor: 'instance creation' stamp: 'gk 11/18/2003 13:44'!
clear
	"Clear out the model in the image. This will forget
	about what packages are installed and what versions.
	The map is itself on disk though and will be reloaded.
	
	If you only want to reload the map and not forget about
	installed packages then use 'SMSqueakMap default reload'.

	If you want to throw out the map perhaps when shrinking
	an image, then use 'SMSqueakMap default purge'."

	"SMSqueakMap clear"

	DefaultMap _ nil! !

!SMSqueakMap class methodsFor: 'instance creation' stamp: 'gh 10/22/2002 14:14'!
default
	"Return the default map, create one if missing."

	"SMSqueakMap default"

	^DefaultMap ifNil: [DefaultMap _ self new]! !

!SMSqueakMap class methodsFor: 'instance creation' stamp: 'gh 10/21/2002 15:05'!
defaultNoCreate
	"Return the default map or nil if there is none."

	"SMSqueakMap defaultNoCreate"

	^DefaultMap! !

!SMSqueakMap class methodsFor: 'discarding' stamp: 'gk 7/14/2003 15:04'!
discardSM
	"Discard SqueakMapBase. All the map state is kept in
	the class var DefaultMap in SMSqueakMap and is thus also removed."

	"SMSqueakMap discardSM"

	SystemOrganization removeCategoriesMatching: 'SM-domain'.! !

!SMSqueakMap class methodsFor: 'server detection' stamp: 'gk 10/1/2003 10:38'!
findServer
	"Go through the list of known master servers, ping 
	each one using simple http get on a known 'ping'-url 
	until one responds return the server name. 
	If some servers are bypassed we write that to Transcript. 
	If all servers are down we inform the user and return nil."

	| notAnswering deafServers |
	Socket initializeNetwork.
	notAnswering _ OrderedCollection new.
	Cursor wait
		showWhile: [ServerList
				do: [:server | (self pingServer: server)
						ifTrue: [notAnswering isEmpty
								ifFalse: [deafServers _ String
												streamContents: [:str | notAnswering
														do: [:srvr | str nextPutAll: srvr printString;
																 nextPut: Character cr]].
									Transcript show: ('These SqueakMap master servers did not respond:\' , deafServers , 'Falling back on ' , server printString , '.') withCRs].
							^ server]
						ifFalse: [notAnswering add: server]]].
	deafServers _ String
				streamContents: [:str | notAnswering
						do: [:srvr | str nextPutAll: srvr printString;
								 nextPut: Character cr]].
	self error: ('All SqueakMap master servers are down:\' , deafServers , '\ \Can not update SqueakMap...') withCRs.
	^ nil! !

!SMSqueakMap class methodsFor: 'class initialization' stamp: 'gk 1/19/2004 09:20'!
initialize
	"Initialize the list of master servers.
	If the first three don't answer I just added the Bluefish
	server so that I (Gran Krampe) could get it up and
	running myself if needed."

	"self initialize"

	ServerList _ #('map1.squeakfoundation.org' 'map2.squeakfoundation.org' 'map.squeak.org' 'anakin.bluefish.se')! !

!SMSqueakMap class methodsFor: 'instance creation' stamp: 'gh 10/22/2002 17:41'!
new
	"Create a new server in a new directory
	under the default directory called 'sm'."

	^super new initializeOn: 'sm'! !

!SMSqueakMap class methodsFor: 'instance creation' stamp: 'gh 8/15/2002 07:02'!
newIn: directoryName
	"Create a new server in a new directory
	under the default directory called <directoryName>."

	^super new initializeOn: directoryName! !

!SMSqueakMap class methodsFor: 'changelog replay' stamp: 'rbb 2/18/2005 13:28'!
noteInstalledPackage: uuidString version: version
	"We are replaying a change that indicates that a package
	was just installed. If there is a map we let it record this,
	otherwise we ask the user if we should create/recreate the map."

	| choice |
	DefaultMap
		ifNotNil: [DefaultMap noteInstalledPackage: uuidString version: version]
		ifNil: [
			[choice _ UIManager default chooseFrom: #('Yes' 'No' 'More info')
				title:
'There is no SqueakMap in this image,
do you wish to create/recreate it? (typical answer is Yes)' .
			choice = 3] whileTrue: [self inform:
'When packages are installed using SqueakMap a little mark is made
in the change log. When an image is reconstructed from the changelog
these marks are intended to keep your map informed about what packages
are installed. You probably already have a map on disk which will automatically be
reloaded if you choose ''Yes'', otherwise an new empty map will be created.
If you choose ''No'', it will only result in that SqueakMap will not know that this package
is installed in your image.
If you are still unsure - answer ''Yes'' since that is probably the best.'].
			choice = 1
				ifTrue:[self default noteInstalledPackage: uuidString version: version]]! !

!SMSqueakMap class methodsFor: 'changelog replay' stamp: 'gk 7/29/2003 01:01'!
noteInstalledPackage: uuidString version: version atSeconds: sec number: num
	"We are replaying a change that indicates that a package
	was just installed. If there is a map we let it record this,
	otherwise we ask the user if we should create/recreate the map."

	DefaultMap
		ifNotNil: [DefaultMap noteInstalledPackage: uuidString version: version
					atSeconds: sec number: num]
		ifNil: [
			self askUser
				ifTrue:[self default noteInstalledPackage: uuidString version: version
							atSeconds: sec number: num]]! !

!SMSqueakMap class methodsFor: 'changelog replay' stamp: 'gk 11/17/2003 01:20'!
noteInstalledPackageWithId: uuidString autoVersion: version atSeconds: sec number: num
	"We are replaying a change that indicates that a package release
	was just installed using SM2. If there is a map we let it record this,
	otherwise we ask the user if we should create/recreate the map."

	DefaultMap
		ifNotNil: [DefaultMap noteInstalledPackageWithId: uuidString autoVersion: version
					atSeconds: sec number: num]
		ifNil: [
			self askUser
				ifTrue:[self default noteInstalledPackageWithId: uuidString autoVersion: version
							atSeconds: sec number: num]]! !

!SMSqueakMap class methodsFor: 'changelog replay' stamp: 'gk 11/17/2003 01:18'!
noteInstalledPackageWithId: uuidString version: version atSeconds: sec number: num
	"We are replaying a change that indicates that a package release
	was just installed using SM2. If there is a map we let it record this,
	otherwise we ask the user if we should create/recreate the map."

	DefaultMap
		ifNotNil: [DefaultMap noteInstalledPackageWithId: uuidString version: version
					atSeconds: sec number: num]
		ifNil: [
			self askUser
				ifTrue:[self default noteInstalledPackageWithId: uuidString version: version
							atSeconds: sec number: num]]! !

!SMSqueakMap class methodsFor: 'server detection' stamp: 'gk 10/1/2003 10:45'!
pingServer: aServerName
	"Check if the SqueakMap server is responding.
	For an old image we first make sure the name resolves -
	the #httpGet: had such a long timeout (and hanging?)
	for resolving the name."

	| url answer |
	"Only test name lookup first if image is before the network rewrite,
	after the rewrite it works."
	[(SystemVersion current highestUpdate < 5252)
		ifTrue: [NetNameResolver addressForName: (aServerName upTo: $:) timeout: 5].
	url _ 'http://', aServerName, '/sm/ping'.
	answer _ HTTPSocket httpGet: url]
				on: Error do: [:ex | ^false].
	^answer isString not and: [answer contents = 'pong']! !

!SMSqueakMap class methodsFor: 'migration' stamp: 'gk 7/29/2003 01:53'!
recreateInstalledPackagesFromChangeLog
	"Clear and recreate the Dictionary with information on installed packages.

	NOTE: This takes some time to run and will only find packages installed using SM
	and since the last changelog condense.

	For packages installed prior to SqueakMap 1.07 there is no timestamp nor counter
	logged. These packages will be given the time of the replay and a separate count
	(from -10000 upwards) maintaining correct order of installation."

	"SMSqueakMap recreateInstalledPackagesFromChangeLog"

	| changesFile chunk |
	SMSqueakMap default clearInstalledPackages.
	changesFile _ (SourceFiles at: 2) readOnlyCopy.
	[changesFile atEnd]
		whileFalse: [
			chunk _ changesFile nextChunk.
			((chunk beginsWith: '"Installed') and: [
				(chunk indexOfSubCollection: 'SMSqueakMap noteInstalledPackage:'
					startingAt: 10) > 0])
				ifTrue: [Compiler evaluate: chunk logged: false]].
	changesFile close! !

!SMSqueakMap class methodsFor: 'constants' stamp: 'gk 10/10/2003 11:40'!
version
	"This is the protocol version number used for clients to decide if
	they need to update SMSqueakMap before synching with
	the master. In short - only increase this if changes have made
	the clients incompatible so that they need to be updated.

	2.0: Removed Module stuff and added Package releases."

	^'2.0'! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gk 8/4/2003 11:50'!
accountForEmail: email
	"Find account given email."

	^self accounts detect: [:a | a email = email] ifNone: [nil]! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gk 8/5/2003 10:59'!
accountForName: name
	"Find account given full name. Disregarding case
	and allows up to 2 different characters.
	Size must match though, someone else can be smarter -
	this is just for migrating accounts properly."

	| lowerName size aName |
	lowerName _ name asLowercase.
	size _ lowerName size.
	^self accounts
		detect: [:a |
			aName _ a name asLowercase.
			(aName size = size) and: [| errors |
				errors _ 0.
				aName with: lowerName do: [:c1 :c2 |
					c1 ~= c2 ifTrue: [errors _ errors + 1]].
				errors < 3
			]]
		ifNone: [nil]
		! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gk 7/30/2003 14:11'!
accountForUsername: username
	"Find account given username. The username used
	is the developer initials of the account."

	^self users at: username ifAbsent: [nil]! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gk 8/4/2003 16:30'!
accountWithId: anIdString 
	"Look up an account. Return nil if missing.
	Raise error if it is not an account."

	| account |
	account _ self objectWithId: anIdString.
	account ifNil: [^nil].
	account isAccount ifTrue:[^account].
	self error: 'UUID did not map to a account.'! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gk 2/7/2004 14:55'!
accountWithName: aName
	"Look up an account by name. Return nil if missing."

	^self accounts values detect: [:a | a name = aName ] ifNone: [nil]! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gk 8/3/2003 23:35'!
accounts
	"Lazily maintain a cache of all known account objects."

	accounts ifNotNil: [^accounts].
	accounts _ objects select: [:o | o isAccount].
	^accounts! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gk 8/4/2003 17:03'!
accountsByInitials
	"Return the accounts sorted by the developer initials."

	^(SortedCollection sortBlock: [:x :y | x initials <= y initials])
		addAll: self accounts;
		yourself! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gk 8/4/2003 17:00'!
accountsByName
	"Return the accounts sorted by their name."

	^(SortedCollection sortBlock: [:x :y | x name <= y name])
		addAll: self accounts;
		yourself! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gh 12/1/2002 19:30'!
addCard: aCard
	"Deprecated. Add a new card.
	Log it in the logfile and register it in the cards."

	^self addObject: aCard! !

!SMSqueakMap methodsFor: 'public-master' stamp: 'gh 8/15/2002 08:34'!
addCategory: aCategory
	"Add a new category.
	Log it in the logfile."

	self log: aCategory.
	^categories at: aCategory id put: aCategory! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gh 11/28/2002 20:46'!
addCategory: category inCard: card
	"Deprecated. Add a category in a card.
	Log the addition."

	^self addCategory: category inObject: card
! !

!SMSqueakMap methodsFor: 'public-master' stamp: 'gk 11/17/2003 10:30'!
addCategory: category inObject: object
	"Add a category in an object."

	^object addCategory: category
! !

!SMSqueakMap methodsFor: 'transactions' stamp: 'gk 10/10/2003 11:30'!
addDirty: anSMObject
	"Add the SMObject to the dirty list making
	sure it gets committed when transaction ends."

"In first SM2 version we do nothing"

"	dirtyList add: anSMObject"! !

!SMSqueakMap methodsFor: 'public-master' stamp: 'gk 2/7/2004 14:56'!
addObject: anSMObject 
	"Add a new object, only if not already added."

	(self object: anSMObject id) ifNil: [
		self transaction: [self newObject: anSMObject]]! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gk 10/21/2003 23:05'!
adminPassword: aString
	"We store the password as a SHA hash so that we can let the slave maps
	have it too."

	adminPassword _ SecureHashAlgorithm new hashMessage: aString! !

!SMSqueakMap methodsFor: 'public-packages' stamp: 'gk 11/14/2003 12:06'!
allPackages
	"Answer all packages."

	^self packages! !

!SMSqueakMap methodsFor: 'public-packages' stamp: 'gh 12/1/2002 19:53'!
availablePackages
	"Answer all packages that are old or not installed."

	^self packages select: [:package | package isAvailable]! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'btr 5/28/2003 02:16'!
cache
	^ fileCache! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gh 3/15/2003 18:12'!
cardWithId: anIdString 
	"Deprecated - Look up a card. Return nil if missing."

	^self packageWithId: anIdString! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gh 11/28/2002 21:59'!
cardWithName: aName 
	"Look up a package by exact match on name. Return nil if missing."

	^self packageWithName: aName! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gh 11/28/2002 20:58'!
cardWithNameBeginning: aString
	"Deprecated. Look up a package beginning with <aString>. Return nil if missing.
	We return the shortest matching one. We also strip out spaces and
	ignore case in both <aString> and the names."

	^self packageWithNameBeginning: aString! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gh 12/1/2002 19:54'!
cards
	^self packages! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gh 11/28/2002 20:55'!
cardsByName
	"Deprecated. Return the cards sorted by their name."

	^self packagesByName! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gh 3/15/2003 18:01'!
categories
	"Lazily maintain a cache of all known category objects."

	categories ifNotNil: [^categories].
	categories _ objects select: [:o | o isCategory].
	^categories! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gh 12/1/2002 20:10'!
categoryWithId: anIdString 
	"Look up a category. Return nil if missing.
	Raise error if it is not a category."

	| cat |
	cat _ self objectWithId: anIdString.
	cat ifNil: [^nil].
	cat isCategory ifTrue:[^cat].
	self error: 'UUID did not map to a category.'! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gk 11/11/2003 18:31'!
categoryWithNameBeginning: aString
	"Look up a category beginning with <aString>. Return nil if missing.
	We return the shortest matching one. We also strip out spaces and
	ignore case in both <aString> and the names."

	| candidates shortest answer searchString |
	searchString _ (aString asLowercase) copyWithout: Character space.
	candidates _ self categories select: [:cat |
		((cat name asLowercase) copyWithout: Character space)
			beginsWith: searchString ].
	shortest _ 1000.
	candidates do: [:ca |
		ca name size < shortest ifTrue:[answer _ ca. shortest _ ca name size]].
	^answer	! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gh 8/13/2002 14:19'!
changeCategoriesTo: newCategories inCard: card
	"Remove or add categories in a card such that
	it belongs to the categories in <newCategories>.
	Logs the changes."

	newCategories do: [:cat |
		(card hasCategory: cat)
			ifFalse:[self addCategory: cat inCard: card]].
	card categories do: [:cat |
		(newCategories includes: cat)
			ifFalse: [self removeCategory: cat inCard: card]]
! !

!SMSqueakMap methodsFor: 'public-master' stamp: 'gh 11/28/2002 20:44'!
changeCategoriesTo: newCategories inObject: object
	"Remove or add categories in an object such that
	it belongs to the categories in <newCategories>.
	Logs the changes."

	newCategories do: [:cat |
		(object hasCategory: cat)
			ifFalse:[self addCategory: cat inObject: object]].
	object categories do: [:cat |
		(newCategories includes: cat)
			ifFalse: [self removeCategory: cat inObject: object]]
! !

!SMSqueakMap methodsFor: 'private' stamp: 'gh 10/31/2002 11:59'!
checkVersion: string
	"Check the content for a SqueakMap version conflict notification.
	Return true if no conflict is reported, otherwise ask user if we
	should upgrade SqueakMap using the bootstrap method."

	(string beginsWith: 'Server version:')
		ifTrue:[(self confirm: ('The SqueakMap master server is running another version (', (string last: (string size - 15)), ') than the client (', SMSqueakMap version, ').
You need to upgrade the SqueakMap package, would you like to do that now?'))
			ifTrue: [self class bootStrap. ^false]
			ifFalse: [^false]
	].
	^true! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gk 10/21/2003 23:02'!
checkpointNumber

	^checkpointNumber! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 11/18/2003 00:18'!
clearCaches
	"Clear the caches."

	packages _ accounts _ users _ categories _ nil
! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 8/8/2003 19:09'!
clearCachesFor: anObject 
	"Clear the valid caches."

	anObject isPackage ifTrue:[packages _ nil].
	anObject isAccount ifTrue:[accounts _ users _ nil].
	anObject isCategory ifTrue:[categories _ nil]
! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/19/2003 19:51'!
clearInstalledPackageWithId: aPackageId
	"Clear the fact that any release of this package is installed.
	Can be used even when the map isn't loaded."

	
^installedPackages ifNotNil: [
		installedPackages removeKey: (UUID fromString: aPackageId) ifAbsent: [nil]]! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/17/2003 01:23'!
clearInstalledPackages
	"Simply clear the dictionary with information on installed packages.
	Might be good if things get corrupted etc. Also see
	SMSqueakMap class>>recreateInstalledPackagesFromChangeLog"

	installedPackages _ nil.
	installCounter _ 0! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 8/8/2003 19:08'!
clearUsernames
	"Clear the username cache."

	users _ nil! !

!SMSqueakMap methodsFor: 'checkpoints' stamp: 'gk 11/3/2003 23:59'!
compressFile: aFileStream
	"Shamelessly copied and modified from StandardFileStream>>compressFile."
	
	| zipped buffer |
	aFileStream binary.
	zipped _ self directory newFileNamed: (aFileStream name, 'gz').
	zipped binary; setFileTypeToObject.
	"Type and Creator not to be text, so can be enclosed in an email"
	zipped _ GZipWriteStream on: zipped.
	buffer _ ByteArray new: 50000.
	[[aFileStream atEnd] whileFalse: [
		zipped nextPutAll: (aFileStream nextInto: buffer)]]
		ensure: [zipped close.aFileStream close].
	self directory deleteFileNamed: aFileStream name! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 11/14/2003 14:01'!
copyFrom: aMap
	"Copy all relevant info from the other map."

	objects _ aMap objects.
	objects do: [:o | o map: self].
	accounts _ users _ packages _ categories _ nil.
	checkpointNumber _ aMap checkpointNumber.! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 7/14/2003 23:52'!
countInstall
	"Increase the install counter."

	installCounter ifNil: [installCounter _ 0].
	^installCounter _ installCounter + 1
! !

!SMSqueakMap methodsFor: 'checkpoints' stamp: 'gk 10/10/2003 14:47'!
createCheckpoint
	"Export a new checkpoint of me using an ImageSegment."

	^self createCheckpointNumber: 
		(self nextFileNameForCheckPoint findTokens: '.') second asNumber.
! !

!SMSqueakMap methodsFor: 'checkpoints' stamp: 'nk 8/21/2004 19:10'!
createCheckpointNumber: number
	"Export me using an ImageSegment.
	This is used for checkpointing the map on disk
	in a form that can be brought into an independent image.
	We do not overwrite older versions, since using ImageSegments
	is an intermediate hack anyway we don't care about the disk waste!!
	Sidenote: Some refactoring was needed to produce a .gz file directly so
	I didn't bother."

	| is fname stream oldMutex |
	fname _ self filename, '.', number asString, '.s'.
	(self directory fileExists: fname) ifTrue: [self error: 'Checkpoint already exists!!'].
	stream _ StandardFileStream newFileNamed: (self directory fullNameFor: fname).
	checkpointNumber _ number.
	oldMutex _ mutex.
	mutex _ nil. self clearCaches.
	[is _ ImageSegment new.
	is copyFromRoots: (Array with: self) sizeHint: 1000000 areUnique: true.
	is writeForExportOn: stream.
	self compressFile: (StandardFileStream oldFileNamed: (self directory fullNameFor: fname)).
	isDirty _ false]
		ensure: [mutex _ oldMutex].
	^is! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 8/15/2002 07:33'!
createNewLog
	"Create a new empty logfile. First chunk is current date."

	^self createNewLogWithInitialContent: nil! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gk 10/15/2003 12:35'!
createNewLogWithInitialContent: content
	"Create a new logfile. First chunk is current date.
	Fill it with optional content."

	| file directory |
	directory _ self directory.
	[file _ directory newFileNamed: (directory nextNameFor: 'squeakmap' extension: 'log').
	file nextChunkPut: Date today storeString.
	content ifNotNil: [file nextPutAll: content]]
		ensure: [file close]! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gh 11/27/2002 12:46'!
created: c updated: u name: n summary: s url: ur
modulePath: mp registrator: r password: p
	"Deprecated. Only kept for migration from SM 1.0x.
	Method used when recreating from storeOn: format."

	^self! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gh 3/19/2002 23:06'!
daysBacklog
	^daysBacklog! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gh 3/19/2002 23:06'!
daysBacklog: days
	daysBacklog _ days! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gh 11/28/2002 21:44'!
deleteCardWithId: anIdString
	"Deprecated. Delete a card."

	^self deleteObjectWithId: anIdString
! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 11/14/2003 11:53'!
deleteCategory: cat
	"Delete a category. Remove it and all its subCategories
	from all objects. Remove the category from its parent.
	Finally remove them all from my collections."

	cat removeDeepFromObjects.
	cat removeFromParent.
	cat allCategoriesDo: [:c |
		categories removeKey: c id ].
	^cat
! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gh 11/28/2002 22:14'!
deleteCategoryWithId: anIdString
	"Deprecated. Delete a category."

	^self deleteObjectWithId: anIdString
! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 8/8/2003 19:10'!
deleteObject: anObject 
	"Delete an object, remove it from objects.
	This method is called from the #delete method of
	anObject so it will take care of the rest of the
	cleaning up. Clear the valid caches."

	objects removeKey: anObject id.
	self clearCachesFor: anObject
! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gk 8/8/2003 10:00'!
deleteObjectWithId: anIdString
	"Delete an object. Polymorphic message."

	^(self objectWithId: anIdString) delete
! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gk 5/22/2004 21:56'!
directory
	"Return the subdirectory that SqueakMap uses."
	
	(FileDirectory default directoryExists: dir)
		ifFalse:[FileDirectory default createDirectory: dir].
	^FileDirectory default directoryNamed: dir! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 8/4/2003 15:22'!
emailOccupied: aUsername
	"Return true if email already taken."

	^(self accountForEmail: aUsername) notNil! !

!SMSqueakMap methodsFor: 'checkpoints' stamp: 'gk 10/10/2003 12:07'!
extension
	^'sgz'! !

!SMSqueakMap methodsFor: 'checkpoints' stamp: 'gk 10/10/2003 12:07'!
filename
	^'map'! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 8/8/2003 02:09'!
findOrCreatePublisher: pub password: password package: aPackage
	"Find or create an account. Password will be the first one."

	| email name account |
	email _ SMUtilities stripEmailFrom: pub.
	name _ SMUtilities stripNameFrom: pub.
	account _ self accountForEmail: email.
	account ifNil: ["none found, try searching on name too..."
		account _ self accountForName: name].
	account
		ifNotNil: [account addObject: aPackage]
		ifNil: ["Ok, create it"	
			account _ self newAccount
					name: name;
					email: email;
					password: password.
			account addObject: aPackage.
			self newObject: account.
			account].
	^account
	
! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gh 8/2/2002 14:26'!
firstTransactionNumber
	^firstTransactionNumber! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gh 8/2/2002 14:28'!
firstTransactionNumber: aCount
	"Set the first known transaction number."

	firstTransactionNumber _ transactionCounter _ aCount
! !

!SMSqueakMap methodsFor: 'checkpoints' stamp: 'nk 8/2/2004 13:27'!
getLastCheckpointWithFilename
	"Return a readstream on a fresh checkpoint gzipped imagesegment.
	First we check if we are dirty and must create a new checkpoint.
	The filename is tacked on at the end so that the checkpoint number
	can be used on the client side too."

	| directory fname |
	isDirty ifTrue: [self createCheckpoint].
	directory _ self directory.
	fname _ self lastCheckpointFilename.
	fname ifNil: [self error: 'No checkpoint available'].
	^((directory oldFileNamed: fname)
		converter: Latin1TextConverter new;
	 	contentsOfEntireFile), ':', fname! !

!SMSqueakMap methodsFor: 'initialize-release' stamp: 'gk 5/22/2004 21:59'!
initializeOn: directoryName
	"Create the local directory for SqueakMap."

	dir _ directoryName.
	(FileDirectory default directoryExists: dir)
		ifFalse:[FileDirectory default createDirectory: dir].
	fileCache _ SMFileCache newFor: self.
	checkpointNumber _ 1.
	daysBacklog _ 182. "about 6 months"! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 7/14/2004 15:51'!
installPackage: aPackage
	"Install the package.

	Note: This method should not be used anymore, better
	to specify a specific release."

	| rel |
	rel _ aPackage lastPublishedReleaseForCurrentSystemVersion
			ifNil: [self error: 'No published release for this system version found to install.'].
	^self installPackageRelease: rel! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 1/15/2004 00:14'!
installPackage: aPackage autoVersion: version
	"Install the release <version> of <aPackage.
	<version> is the automatic version name."

	| r |
	r _ aPackage releaseWithAutomaticVersionString: version.
	r ifNil: [self error: 'No package release found with automatic version ', version].
	^self installPackageRelease: r! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 2/12/2004 23:41'!
installPackageLatestPublished: aPackage
	"Install the latest published release of the package."

	^self installPackageRelease: aPackage lastPublishedRelease! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 2/12/2004 23:40'!
installPackageLatestPublishedNamed: aString
	"Install the latest published release of the package with a name
	beginning with aString (see method comment
	of #packageWithNameBeginning:)."

	| p |
	p _ self packageWithNameBeginning: aString.
	p ifNil: [self error: 'No package found with name beginning with ', aString].
	^self installPackageLatestPublished: p! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 7/14/2004 15:54'!
installPackageNamed: aString
	"Install the last published release
	for this Squeak version of the package with a name
	beginning with aString (see method comment
	of #packageWithNameBeginning:).

	Note: This method should not be used anymore.
	Better to specify a specific release."

	| p |
	p _ self packageWithNameBeginning: aString.
	p ifNil: [self error: 'No package found with name beginning with ', aString].
	^self installPackage: p! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/17/2003 23:35'!
installPackageNamed: aString autoVersion: version
	"Install the release <version> of the package with a name
	beginning with aString (see method comment
	of #packageWithNameBeginning:). <version> is the
	automatic version name."

	| p r |
	p _ self packageWithNameBeginning: aString.
	p ifNil: [self error: 'No package found with name beginning with ', aString].
	r _ p releaseWithAutomaticVersionString: version.
	r ifNil: [self error: 'No package release found with automatic version ', version].
	^self installPackageRelease: r! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 7/14/2004 15:57'!
installPackageRelease: aPackageRelease
	"Install the given package release, no checks made."

	(SMInstaller forPackageRelease: aPackageRelease) install! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/17/2003 23:24'!
installPackageReleaseWithId: anUUIDString
	"Look up and install the given release."

	| r |
	r _ self packageReleaseWithId: anUUIDString.
	r ifNil: [self error: 'No package release available with id: ''', anUUIDString, ''''].
	^self installPackageRelease: r! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/17/2003 23:30'!
installPackageWithId: anUUIDString
	"Look up and install the latest release of the given package.	

	Note: This method should not be used anymore.
	Better to specify a specific release."

	| package |
	package _ self packageWithId: anUUIDString.
	package ifNil: [self error: 'No package available with id: ''', anUUIDString, ''''].
	^self installPackage: package! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 1/15/2004 00:15'!
installPackageWithId: anUUIDString autoVersion: version
	"Install the release <version> of the package with id <anUUIDString>.
	<version> is the automatic version name."

	| p |
	p _ self packageWithId: anUUIDString.
	p ifNil: [self error: 'No package available with id: ''', anUUIDString, ''''].
	^self installPackage: p autoVersion: version! !

!SMSqueakMap methodsFor: 'public-packages' stamp: 'gh 12/1/2002 19:54'!
installableAndNotInstalledPackages
	"Answer all installable but not installed packages."

	^self packages select: [:package | package isInstallableAndNotInstalled]! !

!SMSqueakMap methodsFor: 'public-packages' stamp: 'gh 12/1/2002 19:54'!
installablePackages
	"Answer all packages that can be (auto)installed -
	we have installers that can install them."

	^self packages select: [:package | package isInstallable]! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 1/18/2004 15:29'!
installedPackageReleases
	"Answer all package releases that we know are installed.
	Lazily initialize. The Dictionary contains the installed packages
	using their UUIDs as keys and the version string as the value."

	^self installedPackages collect: [:p | self installedReleaseOf: p]! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 2/17/2004 13:05'!
installedPackages
	"Answer all packages that we know are installed.
	Lazily initialize. The Dictionary contains the installed packages
	using their UUIDs as keys and the version string as the value."

	| result p |
	result _ OrderedCollection new.
	installedPackages ifNil: [#()]
		ifNotNil: [installedPackages keys
					do: [:k |
						p _ self object: k.
						p ifNotNil: [result add: p]]].
	^result! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 7/14/2003 23:35'!
installedPackagesDictionary
	"Access the dictionary directly. The UUID of the installed package is the key.
	The value is an OrderedCollection of Arrays.
	The arrays have the smartVersion of the package, the time of the
	installation in seconds and the sequence number (installCounter)."

	^installedPackages ifNil: [Dictionary new]! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/19/2003 20:05'!
installedPackagesDictionary: aDict
	"Set dictionary directly."

	installedPackages _ aDict! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/19/2003 21:02'!
installedReleaseOf: aPackage
	"If the package is installed, return the release.
	Otherwise return nil. SM2 stores the version as
	an Association to be able to distinguish it."

	| autoVersionOrOld |
	installedPackages ifNil: [^nil].
	autoVersionOrOld _ (installedPackages at: aPackage id ifAbsent: [^nil]) last first.
	(autoVersionOrOld isKindOf: Association)
		ifTrue: [
			^aPackage releaseWithAutomaticVersion: autoVersionOrOld value]
		ifFalse: [
			^aPackage releaseWithVersion: autoVersionOrOld]! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 1/23/2004 09:55'!
installedVersionOf: aPackage
	"If the package is installed, return the version as a String.
	If it is a package installed during SM1 it will return the manual version String,
	for SM2 it returns the automatic version as a String.
	If package is not installed - return nil. If you want it to work without the map loaded you
	should instead use #installedVersionOfPackageWithId:."

	| versionOrString |
	versionOrString _ self installedVersionOfPackageWithId: aPackage id.
	versionOrString ifNil: [^nil].
	^versionOrString isString
		ifTrue: [versionOrString]
		ifFalse: [versionOrString versionString]! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/19/2003 21:10'!
installedVersionOfPackageWithId: anId
	"If the package is installed, return the automatic version or version String.
	Otherwise return nil. This can be used without the map loaded."

	| autoVersionOrOld |
	installedPackages ifNil: [^nil].
	autoVersionOrOld _ (installedPackages at: anId ifAbsent: [^nil]) last first.
	(autoVersionOrOld isKindOf: Association)
		ifTrue: [
			^autoVersionOrOld value]
		ifFalse: [
			^autoVersionOrOld]! !

!SMSqueakMap methodsFor: 'checkpoints' stamp: 'gk 10/10/2003 15:02'!
isCheckpointAvailable
	"Check that there is an 'sm' directory
	and that it contains at least one checkpoint."

	[^self lastCheckpointFilename notNil] on: Error do: [:ex | ^false]! !

!SMSqueakMap methodsFor: 'transactions' stamp: 'gk 11/4/2003 00:19'!
isDirty
	"Is the map modified but not yet checkpointed to disk?"

	^isDirty! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 10/2/2003 10:24'!
isLogFileAvailable
	"Check that there is an 'sm' directory
	and that it contains at least one logfile."

	[self logFileName] on: Error do: [:ex | ^false].
	^true! !

!SMSqueakMap methodsFor: 'checkpoints' stamp: 'gk 10/10/2003 14:21'!
lastCheckpointFilename
	"Return the filename for the newest checkpoint."

	^self directory lastNameFor: self filename extension: self extension! !

!SMSqueakMap methodsFor: 'checkpoints' stamp: 'gk 10/10/2003 14:52'!
lastCheckpointNumberOnDisk
	"Return the last checkpoint number on disk."

	^(self nextFileNameForCheckPoint findTokens: '.') second asNumber - 1! !

!SMSqueakMap methodsFor: 'private' stamp: 'ls 8/23/2003 16:01'!
lastTransactionInLog
	"Find the last transaction number in the current logfile."

	| file |
	[ file _ self openLogFileReadOnly setToEnd.
	#('self transactionCounter:' 'self firstTransactionNumber:')
		do: [:key |
			(file findStringFromEnd: key) = 0
				ifFalse: [
					file skip: key size.
					^(file upTo: $!!) withBlanksTrimmed asNumber ]]
	] ensure: [file close].
	^0 "No marker found"
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 19:30'!
loadDateFrom: aStream
	"Load the creation date from the first chunk."

	^Compiler evaluate: aStream nextChunk
! !

!SMSqueakMap methodsFor: 'public' stamp: 'gh 10/21/2002 14:08'!
loadFull
	"Go through the list of known master servers, ping
	each one using simple http get on a known 'ping'-url
	until one responds and then load the full map from it."
 
	self loadUpdatesFull: true! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 11/17/2003 23:15'!
loadFullFrom: aServerName
	"Contact the SqueakMap at the url <aSqueakMapUrl>
	and load a full map from scratch."

	| url  zipped |
	url _ 'http://', aServerName, '/sm/loadgz?mapversion=', SMSqueakMap version, '&checkpoint=', checkpointNumber asString.
	Transcript show: 'Fetch: ', (Time millisecondsToRun: [ zipped _ (HTTPSocket httpGet: url) contents]) asString, ' ms';cr.
	Transcript show: 'Size: ', zipped size asString, ' bytes';cr.
	((self checkVersion: zipped) and: [zipped ~= 'UPTODATE'])
		ifTrue:[
			Transcript show: 'Save checkpoint to disk: ', (Time millisecondsToRun: [
			self saveCheckpoint: zipped]) asString, ' ms';cr.
			Transcript show: 'Full reload from checkpoint: ', (Time millisecondsToRun: [
			self reload]) asString, ' ms';cr.]! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gk 6/4/2003 21:32'!
loadLog
	"Pick the current logfile - if there is none, create a new and return.
	Otherwise we load the creation date (stored in the beginning).
	Then load all cards in it and build the model. If the creation date is older than 6 months
	we recreate a new logfile in a condensed form and rotate to that."
	
	| file creationDate |
	[file _ self openLogFile.
	file ifNil: [^self createNewLog].
	creationDate _ self loadDateFrom: file.
	self loadUpdatesFrom: file log: false. "we don't log since we are reading from it!!"
	(creationDate < (Date today subtractDays: daysBacklog))
		ifTrue:[self saveNewLog]] ensure: [file ifNotNil: [file close]]! !

!SMSqueakMap methodsFor: 'public' stamp: 'gk 11/17/2003 23:16'!
loadUpdates
	"Go through the list of known master servers, ping
	each one using simple http get on a known 'ping'-url
	until one responds and then load updates from it."

	"SM2 starts with using full always"

	self loadFull! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 11/14/2003 00:52'!
loadUpdatesFrom: aServerName
	"Contact the SqueakMap at the url <aServerName>
	and load needed updates from it based on our last known
	transaction number which we send as an urlencoded argument.
	If the master answers 'DO FULL!!' we issue a full load instead.
	If the master answers 'STALE SERVER!!' we notify the user and bail out."

	| url  zipped updates |
	url _ 'http://', aServerName, '/sm/updatesgz?mapversion=', '1.0', '&transaction=', transactionCounter asString.
	Transcript show: 'Fetch: ', (Time millisecondsToRun: [ zipped _ (HTTPSocket httpGet: url) contents]) asString, ' ms';cr.
	Transcript show: 'Size: ', zipped size asString, ' bytes';cr.
	Transcript show: 'Decompress time: ', (Time millisecondsToRun: [updates _ (GZipReadStream on: zipped) upToEnd]) asString, ' ms';cr.
	(self checkVersion: updates)
		ifTrue:[
			updates = 'STALE SERVER!!' ifTrue:[
				self inform: 'Server ', aServerName printString, ' is stale!! Aborting update.'.
				^self].
			updates = 'DO FULL!!' ifTrue:[
				Transcript show: 'Master can not deliver updates this far back, falling back on full load.';cr.
				^self loadFullFrom: aServerName].
			Transcript show: 'Load updates with logging time: ', (Time millisecondsToRun: [self loadUpdatesFrom: (ReadStream on: updates) log: true]) asString, ' ms';cr.]! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 8/2/2002 01:36'!
loadUpdatesFrom: aStream log: doLogging
	"Load updates from the stream.
	This updates the model in memory and if <doLogging>
	is true we also log the updates in the local log file."

	| chunk file |
	[ doLogging ifTrue:[ file _ self openLogFile setToEnd ].
	[aStream atEnd]
		whileFalse: [
			chunk _ aStream nextChunk.
			Compiler evaluate: chunk for: self logged: false.
			doLogging ifTrue: [file cr; nextChunkPut: chunk].
		]] ensure: [
			aStream close.
			doLogging ifTrue:[file close]]! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 11/22/2002 15:02'!
loadUpdatesFromLog
	"Pick the current logfile - if there is none, create a new and return.
	Otherwise check the last recorded transaction and compare it with
	our own counter. If the counter is less (meaning there are new
	transactions in the logfile) we read them into the model thus bringing
	the model in synch with the logfile.
	The implementation uses the new method #findStringFromEnd: which
	should be reasonably fast. If we find no such marker it means that
	this logfile has been condensed and does not have enough transactions
	to return, then we call #reloadLog which will reload the log completely
	and return false.
	Note: This is used because multiple images can share the same logfile
	and the other images may have added transactions to the logfile."
	
	| file key found |
	[ file _ self openLogFileReadOnly.
	file ifNil: [^self createNewLog].
	file setToEnd.
	key _ 'self transactionCounter: ', transactionCounter storeString, '!!'.
	found _ file findStringFromEnd: key.
	found = 0 ifFalse: [file skip: key size. self loadUpdatesFrom: file log: false]]
		ensure: [file ifNotNil: [file close]].
	found = 0 ifTrue:[self reloadLog]! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 10/10/2003 13:25'!
loadUpdatesFull: full
	"Find a server and load updates from it."
 
	| server |
	server _ self class findServer.
	server ifNotNil: [
		self synchWithDisk.
		full ifTrue: [self loadFullFrom: server]
			ifFalse:[self error: 'Not supported yet!!'."self loadUpdatesFrom: server"]]! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 6/26/2002 16:17'!
log: anSMObject
	"Pick the newest logfile available and log the object in it."
	
	| file |
	[ file _ self openLogFile setToEnd.
	anSMObject logOn: file.
	self logIncrementedTransactionCounterOn: file ]
		ensure: [file close]
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 6/26/2002 16:11'!
logDelete: anSMObject
	"Pick the newest logfile available and log the deletion in it."
	
	| file |
	[ file _ self openLogFile setToEnd.
	anSMObject logDeleteOn: file.
	self logIncrementedTransactionCounterOn: file ]
		ensure: [file close]
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gk 9/18/2003 12:30'!
logFileName
	"Pick the newest logfile available."

	^self directory lastNameFor: 'squeakmap' extension: 'log'! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 8/14/2002 08:14'!
logFirstTransactionNumberOn: aStream
	"Log the transaction number as the first available in the logfile being produced."

	aStream cr; nextChunkPut: 'self firstTransactionNumber: ', transactionCounter storeString
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 23:15'!
logIncrementedTransactionCounterOn: aStream
	"Increment the transaction counter and
	log the current number."

	transactionCounter _ transactionCounter + 1.
	self logTransactionCounterOn: aStream
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 3/19/2002 23:16'!
logTransactionCounterOn: aStream
	"Log the current transaction number."

	aStream cr; nextChunkPut: 'self transactionCounter: ', transactionCounter storeString
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 12/1/2002 21:33'!
logUpdate: anSMObject original: original
	"Pick the newest logfile available and log the updated object in it.
	<original> should be an editCopy created before the changes."
	
	| file |
	[ file _ self openLogFile setToEnd.
	anSMObject logUpdateOn: file original: original.
	self logIncrementedTransactionCounterOn: file ]
		ensure: [file close]
! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 11/17/2003 22:57'!
mandatoryCategoriesFor: aClass
	"Return the categories that are mandatory for instances of <aClass>."

	^self categories select: [:c | c mandatoryFor: aClass]! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gk 8/5/2003 12:41'!
mapInitialsFromMinnow
	"Try to find the developer initials from the table at minnow."

	| page end start table startString stream coll s e sub stripped dict ini problems |
	startString _ '<td> Initials </td></tr>'.
	page _ 'http://minnow.cc.gatech.edu/squeak/8' asUrl retrieveContents content.
	start _ page findString: startString.
	start _ start + startString size.
	end _ page findString: '</table>' startingAt: start.
	table _ page copyFrom: start to: end.
	stream _ ReadStream on: table.
	coll _ OrderedCollection new.
	[stream match: '<td>'.
	s _ stream position.
	stream match: '</td>'] whileTrue: [
		e _ stream position.
		sub _ table copyFrom: s + 1 to: e - 5.
		stripped _ String streamContents: [:str | | skipNext |
			skipNext _ false.
			(sub findTokens: '<>' keep: '<>') do: [:token |
				skipNext
					ifTrue: [skipNext _ false]
					ifFalse: [
						token = '<'
							ifTrue: [skipNext _ true]
							ifFalse: [token = '>' ifFalse: [str nextPutAll: token]]]]].
		coll add: stripped withBlanksTrimmed].
	dict _ Dictionary new.
	1 to: coll size - 2 by: 2 do: [:i |
		dict at: (coll at: i) put: (coll at: i + 1)].
	problems _ OrderedCollection new.
	accounts do: [:a |
		ini _ dict at: a name ifAbsent: [nil].
		((ini isNil or: [ini = '-']) or: [(ini allSatisfy: [:c | c isLetter]) not])
			ifTrue: [problems add: a name, ' has now ', (ini ifNil: ['<NONE FOUND>'])]
			ifFalse: [a initials: ini. Transcript show: ini, ' for ', a name;cr]].
	problems asSortedCollection do: [:p | Transcript show: p; cr].

! !

!SMSqueakMap methodsFor: 'private-installation' stamp: 'gk 11/18/2003 17:48'!
markInstalled: uuid version: version time: time counter: num
	"Private. Mark the installation. SM2 uses an Association
	to distinguish the automatic version from old versions."


	| installs |
	installedPackages ifNil: [installedPackages _ Dictionary new].
	installs _ installedPackages at: uuid
				ifAbsent: [installedPackages at: uuid put: OrderedCollection new].
	installs add:
		(Array with: 2->version
				with: time
				with: num)! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gk 11/7/2003 10:26'!
migrate
	"Convert this map to 1.1 format and fix other things.

	Fixing the bug from SM1.x with 'Maturity level' and
	'SqueakAmp' sharing the same UUID."

	| cat |
	cat _ (self categories detect: [:c |
			c name = 'Alpha']) parent.
	cat id: (UUID fromString: 'b9f16f16-4a58-0442-9159-041dd7213070').
	objects at: cat id put: cat. "Wasn't in there before"

	self mapInitialsFromMinnow! !

!SMSqueakMap methodsFor: 'public-master' stamp: 'gk 11/17/2003 14:56'!
moveCategory: category toAfter: categoryBefore inParent: parent
	"Move a category to be listed after <categoryBefore> in <parent>."

	parent move: category toAfter: categoryBefore.
	^category
	
! !

!SMSqueakMap methodsFor: 'public-master' stamp: 'gk 5/22/2004 22:22'!
moveCategory: category toParent: parentCategory
	"Move a category into another parent category."

	parentCategory
		ifNil: [category parent: nil]
		ifNotNil: [parentCategory addCategory: category].
	^category
	
! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gh 10/30/2002 13:48'!
moveCategoryWithId: anIdString toAfterWithId: beforeIdString
	"Move a category to be before another category.
	If beforeIdString is nil then we move it up to be first."

	| cat before |
	cat _ self categoryWithId: anIdString.
	before _ beforeIdString ifNotNil: [self categoryWithId: beforeIdString].
	cat parent move: cat toAfter: before! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gh 8/14/2002 11:24'!
moveCategoryWithId: anIdString toParentWithId: parentIdString
	"Move a category to another parent.
	If parentIdString is nil then we move it up as a top category."

	| cat parent |
	cat _ self categoryWithId: anIdString.
	parentIdString
		ifNil: [cat parent removeCategory: cat.
			"topCategories add: cat"]
		ifNotNil: [
			parent _ self categoryWithId: parentIdString.
			parent addCategory: cat.
			"topCategories remove: cat ifAbsent: [nil]"]
! !

!SMSqueakMap methodsFor: 'transactions' stamp: 'gk 9/25/2003 23:53'!
mutex
	"Lazily initialize the Semaphore."

	^mutex ifNil: [mutex _ Semaphore forMutualExclusion]! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 8/3/2003 23:23'!
newAccount
	"Create a new account."

	^SMAccount newIn: self! !

!SMSqueakMap methodsFor: 'public-master' stamp: 'gk 8/4/2003 16:01'!
newAccount: name username: username email: email
	"Create an account. Checking for previous account should already have been done.
	To add the account to the map, use SMSqueakMap>>addObject:"

	| account |
	account _ self newAccount
					name: name;
					initials: username;
					email: email.
	^account
	
! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gk 11/6/2003 23:54'!
newAccountWithId: anIdString 
	"Create an account and add it to me.
	Clear the accounts & users cache."

	accounts _ users _ nil.
	^self newObject: (SMAccount newIn: self withId: anIdString)! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gh 11/28/2002 20:39'!
newCardWithId: anIdString
	"Deprecated. Create a card and add it to me."

	^self newPackageWithId: anIdString
! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gh 12/1/2002 20:13'!
newCategoryWithId: anIdString 
	"Create a category and add it to me.
	Clear the categories cache."

	categories _ nil.
	^self newObject: (SMCategory newIn: self withId: anIdString)! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gh 11/28/2002 20:40'!
newCategoryWithId: catIdString inCardWithId: cardIdString
	"Deprecated. Add a category to a card.
	Only kept for converting old log. We add all categories
	to the package (as before) and not the release. "

	^self newCategoryWithId: catIdString inObjectWithId: cardIdString
! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gh 11/28/2002 20:25'!
newCategoryWithId: catIdString inObjectWithId: idString
	"Add a category to a categorizable object."

	^(self objectWithId: idString) addCategory: (self categoryWithId: catIdString)
! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 9/26/2003 00:04'!
newObject: anSMObject 
	"Add an SMObject to me. Clear the valid caches."

	self addDirty: anSMObject.
	self clearCachesFor: anSMObject.
	^objects at: anSMObject id put: anSMObject! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 8/3/2003 23:24'!
newPackage
	"Create a new package."

	^SMPackage newIn: self! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gk 8/7/2003 21:26'!
newPackageReleaseWithId: anIdString inPackageWithId: packageIdString
	"Create a package release and add it to the correct package."

	| release |
	release _ SMPackageRelease newIn: self withId: anIdString.
	(self packageWithId: packageIdString) addRelease: release.
	^self newObject: release! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gh 12/1/2002 20:12'!
newPackageWithId: anIdString 
	"Create a package and add it to me.
	Clear the packages cache."

	packages _ nil.
	^self newObject: (SMPackage newIn: self withId: anIdString)! !

!SMSqueakMap methodsFor: 'deprecated log messages' stamp: 'gh 11/27/2002 12:44'!
newRepositoryWithId: anId
	"Deprecated. Only kept for migration from SM 1.0x.
	Method used when recreating from storeOn: format."

	^self! !

!SMSqueakMap methodsFor: 'checkpoints' stamp: 'gk 10/10/2003 12:12'!
nextFileNameForCheckPoint
	"Return the next available filename for a checkpoint."

	^self directory nextNameFor: self filename extension: self extension! !

!SMSqueakMap methodsFor: 'public-packages' stamp: 'gh 12/1/2002 19:54'!
notInstalledPackages
	"Answer all packages that are not installed."

	^self packages reject: [:package | package isInstalled]! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/18/2003 17:49'!
noteInstalled: aPackageRelease
	"The package release was just successfully installed using SM.
	This is the method being called by SM upon a successful installation.

	We record this in our Dictionary of installed package releases
	and log a 'do it' to mark this in the changelog.
	The map used is the default map."

	^self noteInstalledPackageWithId: aPackageRelease package id asString
		autoVersion: aPackageRelease automaticVersion
		name: aPackageRelease package name! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/18/2003 17:50'!
noteInstalledPackage: aPackage autoVersion: aVersion
	"Mark that the package release was just successfully installed.
	Can be used to inform SM of an installation not been done using SM."

	
^self noteInstalledPackageWithId: aPackage id asString
		autoVersion: aVersion
		name: aPackage name! !

!SMSqueakMap methodsFor: 'private-installation' stamp: 'gk 7/29/2003 01:33'!
noteInstalledPackage: uuidString version: version
	"Mark a specific version of a package as installed.
	This method is called when replaying a logged installation
	from before SqueakMap 1.07. Such logged installations lacked
	a timestamp and a count. We take the current time and a
	count starting from -10000 and upwards. This should keep
	the sorting order correct."

	"Find the lowest installed count."
	| lowest |
	lowest _ 0.
	installedPackages ifNotNil: [
		installedPackages valuesDo: [:oc |
			oc do: [:array |
				array last < lowest ifTrue: [lowest _ array last]]]]
		ifNil: [lowest _ -10000].
	lowest negative ifFalse: [lowest _ -10000].
	^self noteInstalledPackage: uuidString version: version
		atSeconds: Time totalSeconds number: lowest + 1! !

!SMSqueakMap methodsFor: 'private-installation' stamp: 'gk 11/18/2003 17:54'!
noteInstalledPackage: uuidString version: version atSeconds: time number: num
	"Mark a package as installed in the Dictionary.
	This method is called when replaying a logged installation.
	<time> is the point in time as totalSeconds of the installation.
	<num> is the installCount of the installation.
	This method is typically called from a doIt in the changelog
	in order to try to keep track of packages installed."

	num negative ifFalse: ["Not an emulated count from prior SM1.07"
		installCounter _ num max: installCounter].
	self markInstalled: (UUID fromString: uuidString) version: version time: time counter: num! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/18/2003 17:51'!
noteInstalledPackageNamed: aString autoVersion: aVersion
	"Mark that the package release was just successfully installed.
	<aVersion> is the automatic version as a String.
	Can be used to inform SM of an installation not been done using SM."

	| p |
	p _ self packageWithNameBeginning: aString.
	p ifNil: [self error: 'No package found with name beginning with ', aString].
	
^self noteInstalledPackage: p autoVersion: aVersion asVersion! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/18/2003 11:59'!
noteInstalledPackageWithId: aPackageId autoVersion: aVersion
	"The package release was just successfully installed.
	Can be used to inform SM of an installation not been
	done using SM, even when the map isn't loaded."

	
^self noteInstalledPackageWithId: aPackageId
		autoVersion: aVersion
		name: '<unknown name>'! !

!SMSqueakMap methodsFor: 'private-installation' stamp: 'gk 11/18/2003 17:47'!
noteInstalledPackageWithId: uuidString autoVersion: version atSeconds: time number: num
	"Mark a package as installed in the Dictionary.
	This method is called when replaying a logged installation.
	<time> is the point in time as totalSeconds of the installation.
	<num> is the installCount of the installation.
	This method is typically called from a doIt in the changelog
	in order to try to keep track of packages installed."

	num negative ifFalse: ["Not an emulated count from prior SM1.07"
		installCounter _ num max: installCounter].
	self markInstalled: (UUID fromString: uuidString) version: version time: time counter: num! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'nk 8/31/2004 09:02'!
noteInstalledPackageWithId: aPackageId autoVersion: aVersion name: aName 
	"The package release was just successfully installed.
	Can be used to inform SM of an installation not been
	done using SM, even when the map isn't loaded.

	We record the fact in our Dictionary of installed packages
	and log a 'do it' to mark this in the changelog.
	The doit helps keeping track of the packages when
	recovering changes etc - not a perfect solution but should help.
	The map used is the default map.
	The id of the package is the key and the value is an OrderedCollection
	of Arrays with the release auto version, the point in time and the current installCounter."

	| time name id v |
	v := aVersion isString ifTrue: [aVersion asVersion] ifFalse: [aVersion].
	aName ifNil: [name := '<unknown package name>'] ifNotNil: [name := aName].
	id := UUID fromString: aPackageId.
	time := Time totalSeconds.
	self countInstall.
	self 
		markInstalled: id
		version: v
		time: time
		counter: installCounter.
	SmalltalkImage current
		logChange: '"Installed ' , name , ' auto version ' , v versionString 
				, '".
(Smalltalk at: #SMSqueakMap ifAbsent: []) ifNotNil:[
	SMSqueakMap noteInstalledPackageWithId: ' 
					, id asString storeString , ' autoVersion: ' 
				, v storeString , ' atSeconds: ' 
				, time asString , ' number: ' 
				, installCounter asString , ']'! !

!SMSqueakMap methodsFor: 'private-installation' stamp: 'gk 1/22/2004 12:01'!
noteUninstalled: aPackageRelease
	"The package release was just successfully uninstalled using SM.
	This is the method being called by SM upon a successful uninstallation.

	We record this in our Dictionary of installed package releases
	and log a 'do it' to mark this in the changelog.
	The map used is the default map."

	^self noteUninstalledPackageWithId: aPackageRelease package id asString
		autoVersion: aPackageRelease automaticVersion
		name: aPackageRelease package name! !

!SMSqueakMap methodsFor: 'private-installation' stamp: 'nk 7/30/2004 17:53'!
noteUninstalledPackageWithId: aPackageId autoVersion: aVersion name: aName 
	"The package release was just successfully uninstalled.
	Can be used to inform SM of an uninstallation not been
	done using SM, even when the map isn't loaded.

	We record the fact in our Dictionary of installed packages
	and log a 'do it' to mark this in the changelog.
	The doit helps keeping track of the packages when
	recovering changes etc - not a perfect solution but should help.
	The map used is the default map.
	The id of the package is the key and the value is an OrderedCollection
	of Arrays with the release auto version, the point in time and the current installCounter."

	| time name id v |
	v := aVersion isString ifTrue: [aVersion asVersion] ifFalse: [aVersion].
	aName ifNil: [name := '<unknown package name>'] ifNotNil: [name := aName].
	id := UUID fromString: aPackageId.
	time := Time totalSeconds.
	self countInstall.	"Used for both installs and uninstalls"
	self 
		clearInstalled: id
		version: v
		time: time
		counter: installCounter.
	SmalltalkImage current
		logChange: '"Uninstalled ' , name , ' auto version ' , v versionString 
				, '".
(Smalltalk at: #SMSqueakMap ifAbsent: []) ifNotNil:[
	SMSqueakMap noteUninstalledPackageWithId: ' 
					, id asString storeString , ' autoVersion: ' 
				, v storeString , ' atSeconds: ' 
				, time asString , ' number: ' 
				, installCounter asString , ']'! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gk 11/18/2003 02:06'!
object: aUUID
	"Look up a categorizable object. Return nil if missing."

	^objects at: aUUID ifAbsent: [nil]! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gh 12/1/2002 19:25'!
objectWithId: anIdString
	"Look up a categorizable object. Return nil if missing."

	^objects at: (UUID fromString: anIdString) ifAbsent: [nil]! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gh 11/28/2002 22:00'!
objects
	^objects! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 11/14/2003 00:53'!
oldLoadFullFrom: aServerName
	"Contact the SqueakMap at the url <aSqueakMapUrl>
	and load a full map from scratch."

	| url  zipped full |
	url _ 'http://', aServerName, '/sm/loadgz?mapversion=', '1.0'.
	Transcript show: 'Fetch: ', (Time millisecondsToRun: [ zipped _ (HTTPSocket httpGet: url) contents]) asString, ' ms';cr.
	Transcript show: 'Size: ', zipped size asString, ' bytes';cr.
	Transcript show: 'Decompress time: ', (Time millisecondsToRun: [full _ (GZipReadStream on: zipped) upToEnd]) asString, ' ms';cr.
	(self checkVersion: full)
		ifTrue:[
			Transcript show: 'Save full log: ', (Time millisecondsToRun: [
			self createNewLogWithInitialContent: full]) asString, ' ms';cr.
			Transcript show: 'Full reload from log: ', (Time millisecondsToRun: [
			self reloadLog]) asString, ' ms';cr.]! !

!SMSqueakMap methodsFor: 'public-packages' stamp: 'gk 7/13/2004 15:12'!
oldPackages
	"Answer all packages that are installed with a
	newer published version for this Squeak version available."

	^self installedPackages select: [:package | package isSafelyOld]! !

!SMSqueakMap methodsFor: 'logging' stamp: 'nk 8/2/2004 13:27'!
openLogFile
	"Pick the newest logfile available and open it."

	| fileName |
	fileName _ self logFileName.
	fileName ifNil: [^nil].
	^(self directory oldFileNamed: fileName)
		converter: Latin1TextConverter new;
		yourself.
! !

!SMSqueakMap methodsFor: 'logging' stamp: 'gh 10/31/2002 11:23'!
openLogFileReadOnly
	"Pick the newest logfile available and open it."

	| fileName |
	fileName _ self logFileName.
	fileName ifNil: [^nil].
	^self directory readOnlyFileNamed: fileName
! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gk 1/25/2004 17:10'!
packageCacheDirectory
	"Return a FileDirectory for the package cache of the map.
	Creates it if it is missing."

	| dirName baseDir |
	dirName _ self packageCacheDirectoryName.
	baseDir _ self directory.
	(baseDir fileOrDirectoryExists: dirName)
		ifFalse:[baseDir createDirectory: dirName].
	^baseDir directoryNamed: dirName! !

!SMSqueakMap methodsFor: 'private' stamp: 'btr 5/28/2003 00:56'!
packageCacheDirectoryName
	"What is the name of the cache directory?"

	^'cache'! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gk 11/17/2003 23:25'!
packageReleaseWithId: anIdString 
	"Look up a package release. Return nil if missing.
	Raise error if it is not a package release."

	| r |
	r _ self objectWithId: anIdString.
	r ifNil: [^nil].
	r isPackageRelease ifTrue:[^r].
	self error: 'UUID did not map to a package release.'! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gh 12/1/2002 19:39'!
packageWithId: anIdString 
	"Look up a package. Return nil if missing.
	Raise error if it is not a package."

	| package |
	package _ self objectWithId: anIdString.
	package ifNil: [^nil].
	package isPackage ifTrue:[^package].
	self error: 'UUID did not map to a package.'! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gh 12/1/2002 19:54'!
packageWithName: aName
	"Look up a package by exact match on name. Return nil if missing."

	^self packages detect: [:package | package name = aName ] ifNone: [nil]! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gh 12/1/2002 19:54'!
packageWithNameBeginning: aString
	"Look up a package beginning with <aString>. Return nil if missing.
	We return the shortest matching one. We also strip out spaces and
	ignore case in both <aString> and the names."

	| candidates shortest answer searchString |
	searchString _ (aString asLowercase) copyWithout: Character space.
	candidates _ self packages select: [:package |
		((package name asLowercase) copyWithout: Character space)
			beginsWith: searchString ].
	shortest _ 1000.
	candidates do: [:package |
		package name size < shortest ifTrue:[answer _ package. shortest _ package name size]].
	^answer	! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gk 11/11/2003 18:37'!
packageWithPI: aPIName
	"Look up a package by exact match on PackageInfo name. Return nil if missing."

	aPIName isEmptyOrNil ifTrue: [^nil].
	^self packages detect: [:package | package packageInfoName = aPIName ] ifNone: [nil]! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gk 11/18/2003 13:40'!
packages
	"Lazily maintain a cache of all known package objects."

	packages ifNotNil: [^packages].
	objects ifNil: [^#()].
	packages _ objects select: [:o | o isPackage].
	^packages! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gh 12/1/2002 19:54'!
packagesByName
	"Return the packages sorted by their name."

	^(SortedCollection sortBlock: [:x :y | x name <= y name])
		addAll: self packages;
		yourself! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 9/30/2003 17:03'!
pingServer: aServerName

	^self class pingServer: aServerName! !

!SMSqueakMap methodsFor: 'public' stamp: 'gk 7/12/2004 17:48'!
purge
	"Clear out the map from memory. Use this to reclaim space,
	no information is lost because it does not remove information
	about what packages are installed, and the map itself is checkpointed
	to disk. Use #reload to get it back from the latest checkpoint on disk."

	objects := accounts := users := packages := categories := nil.
	checkpointNumber _ 0.! !

!SMSqueakMap methodsFor: 'public' stamp: 'nk 7/30/2004 07:09'!
reload
	"Reload the map from the latest checkpoint on disk.
	The opposite of #purge."

	| fname stream map |
	fname _ self lastCheckpointFilename.
	fname ifNil: [self error: 'No checkpoint available!!'].
	stream _ (self directory oldFileNamed: fname) asUnZippedStream.
	stream ifNil: [self error: 'Couldn''t open stream on checkpoint file!!'].
	stream converter: Latin1TextConverter new.
	[map _ (stream reset fileInObjectAndCode) install arrayOfRoots first] ensure: [stream close].
	self copyFrom: map! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gk 7/30/2003 14:12'!
reloadLog
	"First we clear the current content and then we load the log.
	This should not lead to any differences!!"

	users _ nil.
	categories _ nil.
	packages _ nil.
	accounts _ nil.
	objects _ Dictionary new.
	self loadLog! !

!SMSqueakMap methodsFor: 'public-master' stamp: 'gk 9/24/2003 22:54'!
removeCategory: aCategory
	"Remove a category. Same as deleting it but we log it too."
self halt.

	self deleteCategory: aCategory.
	self logDelete: aCategory.
	^aCategory! !

!SMSqueakMap methodsFor: 'public-master' stamp: 'gk 11/17/2003 10:30'!
removeCategory: category inObject: object
	"Remove a category from an object."

	^object removeCategory: category
! !

!SMSqueakMap methodsFor: 'public-master' stamp: 'gk 8/8/2003 10:00'!
removeObject: anObject
	"Delete it and log it in the logfile."

	anObject delete.
	self logDelete: anObject.
	^anObject
! !

!SMSqueakMap methodsFor: 'public-master' stamp: 'gh 8/2/2002 01:32'!
removeRepository: aRepository
	"Log it in the logfile and delete it."
	
	self deleteRepository: aRepository.
	self logDelete: aRepository.
	^aRepository! !

!SMSqueakMap methodsFor: 'checkpoints' stamp: 'yo 8/16/2004 14:23'!
saveCheckpoint: contentWithFilename
	"Save the map checkpoint to disk if it is not there already."

	| file directory sz fname content |
	directory _ self directory.
	sz _ contentWithFilename size.
	fname _ contentWithFilename last: sz - (contentWithFilename lastIndexOf: $:).
	content _ contentWithFilename first: sz - fname size - 1.
	(directory fileExists: fname) ifFalse: [
		["file _ directory newFileNamed: (directory nextNameFor: 'map' extension: 'sgz')."
		file _ directory newFileNamed: fname.
		file converter: Latin1TextConverter new.
		file nextPutAll: content]
			ensure: [file close]]! !

!SMSqueakMap methodsFor: 'deprecated' stamp: 'gh 8/15/2002 07:07'!
saveNewLog
	"Create a new logfile and store the model condensed in it.
	A slave SqueakMap can do this at any time, it is done automatically at least
	once every 6 months. A master SqueakMap should not do this often at all -
	the more seldom it does the longer backlog it has and can feed slaves with incremental
	updates instead of forcing them to a complete download.
	First chunk is current date. Categories are saved first (top down),
	then all the cards followed by the repositories.
	At the end we tack on the last known transactionId."

	| file |
	self createNewLog.
	[ file _ self openLogFile setToEnd.
	self storeOn: file.
	firstTransactionNumber _ transactionCounter]
		ensure: [file close]! !

!SMSqueakMap methodsFor: 'transactions' stamp: 'gk 11/17/2003 14:11'!
setDirty
	"Set the map modified so that it will get written to disk."

	isDirty _ true! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gk 7/13/2004 02:51'!
silent
	"Can installations ask questions or should they be silent
	and us good defaults?"

	^ silent ifNil: [false] ifNotNil: [true]! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 7/13/2004 02:39'!
silentlyDo: aBlock
	"Execute <aBlock> with the Silent flag set.
	This is a crude way of avoiding user interaction
	during batch operations, like loading updates."

	[silent _ true.
	aBlock value]
		ensure: [silent _ nil]! !

!SMSqueakMap methodsFor: 'private' stamp: 'gh 11/28/2002 22:19'!
storeOn: aStream 
	"Store the model condensed on the stream.
	First chunk is current date. Categories are saved first (in a top down
	manner so that loading will be able to resolve parents directly),
	then all the objects. At the end we tack on the last known transactionId."

	self topCategories do: [:cat | cat logRecursivelyOn: aStream].
	objects valuesDo: [:package | package logOn: aStream].
	self logFirstTransactionNumberOn: aStream! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 11/17/2003 23:15'!
synchWithDisk
	"Synchronize myself with the checkpoints on disk.
	If there is a newer checkpoint than I know of, load it.
	If there is no checkpoint or if I have a higher checkpoint number,
	create a new checkpoint from me.

	The end result is that I am in synch with the disk and we are both as
	updated as possible."

	| checkpointNumberOnDisk |
	 "If there is no checkpoint, save one from me."
	(self isCheckpointAvailable) ifFalse: [^self createCheckpointNumber: checkpointNumber].
	"If the one on disk is newer, load it"
	checkpointNumberOnDisk _ self lastCheckpointNumberOnDisk.
	(checkpointNumber < checkpointNumberOnDisk)
		ifTrue: [^self reload].
	"If I am newer, recreate me on disk"
	(checkpointNumberOnDisk < checkpointNumber)
		ifTrue: [^self createCheckpointNumber: checkpointNumber]! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 10/2/2003 10:34'!
synchWithLog
	"Synchronize myself with the current logfile.
	If the logfile has newer updates, load them.
	If the logfile can not produce updates as far back as I need, do a full reload.
	If the logfile does not exist or if I have newer updates than the logfile,
	create a new logfile from me.

	The end result is that I am in synch with the logfile and we are both as
	updated as possible."

	| updates |
	 "If there is no logfile, create it from ourself."
	(self isLogFileAvailable) ifFalse: [self saveNewLog. ^self].
	updates _ self updatesSinceTransactionInLog: transactionCounter.
	updates = '' ifTrue: [^self]. "Already in synch."
	updates = 'DO FULL!!'
		ifFalse: [
			updates = 'STALE SERVER!!'
				ifTrue: [self saveNewLog]
				ifFalse: [self loadUpdatesFrom: (ReadStream on: updates) log: false]]
		ifTrue: [self reloadLog]! !

!SMSqueakMap methodsFor: 'queries' stamp: 'gh 12/1/2002 20:12'!
topCategories
	^self categories select: [:cat | cat isTopCategory]! !

!SMSqueakMap methodsFor: 'transactions' stamp: 'gk 1/17/2004 16:00'!
transaction: aBlock
	"Execute aBlock and then make sure any modified SMObjects
	are committed to disk. We do this inside a mutex in order to
	serialize transactions. Transactions must be initiated from
	service methods in this class and not from inside the domain
	objects - otherwise they could get nested and a deadlock occurs."

"In first version of SM2 we simply set the isDirty flag,
when next client asks for updates, or 30 minutes has passed,
we checkpoint."

"	self mutex critical: ["
		aBlock value.
		isDirty _ true
"	]"

"	self mutex critical: [
		dirtyList _ OrderedCollection new.
		aBlock value.
		dirtyList do: [:obj | obj commit].
		dirtyList _ nil
	]"! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gh 7/31/2002 14:27'!
transactionCounter
	"Get the last known transaction number."

	^transactionCounter
! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gh 3/19/2002 23:08'!
transactionCounter: aCount
	"Set the last known transaction number."

	transactionCounter _ aCount
! !

!SMSqueakMap methodsFor: 'private' stamp: 'gh 8/2/2002 14:31'!
updatesSinceFirstTransaction
	"Produce a String with all recorded transactions from the logfile.
	We go to the beginning of the file,
	then we search for the text 'firstTransactionNumber: xxx'.
	Finally we return all transactions from then on.
	The implementation uses the new method #findString: which
	should be reasonably fast."

	| file found result key |
	[ file _ self openLogFileReadOnly.
	key _ 'self firstTransactionNumber:'.
	found _ file findString: key.
	found = 0
		ifTrue:[result _ nil]
		ifFalse: [
			file skip: key size; upTo: $!!; next.
			result _ file upToEnd ]]
				ensure: [file close].
	^result
! !

!SMSqueakMap methodsFor: 'private' stamp: 'gh 10/21/2002 14:21'!
updatesSinceTransaction: lastTransaction
	"Produce a String with all transactions from the logfile
	since <lastTransaction>. We go to the end of the file,
	then we search backwards for the text 'transactionCounter: xxx'.
	The implementation uses the new method #findStringFromEnd: which
	should be reasonably fast. If we find no such marker it means that
	this logfile has been condensed and does not have enough transactions
	to return, then we return 'DO FULL!!'.
	The client will have to ask for the whole file instead.
	If the server has a lower transaction counter than the one sent
	the server is stale, let the client know this."

	| file found result key |
	lastTransaction = transactionCounter ifTrue:[^''].
	lastTransaction > transactionCounter ifTrue:[^'STALE SERVER!!'].
	lastTransaction = firstTransactionNumber ifTrue:[^self updatesSinceFirstTransaction ].
	[ file _ self openLogFileReadOnly setToEnd.
	key _ 'self transactionCounter: ', lastTransaction storeString, '!!'.
	found _ file findStringFromEnd: key.
	found = 0 ifTrue:[result _ 'DO FULL!!'] ifFalse: [ file skip: key size. result _ file upToEnd ]]
		ensure: [file close].
	^result
! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 7/14/2003 01:21'!
updatesSinceTransactionInLog: lastTransaction
	"Produce a String with all transactions from the logfile
	since <lastTransaction>. We go to the end of the file,
	then we search backwards for the text 'transactionCounter: xxx'.
	The implementation uses the new method #findStringFromEnd: which
	should be reasonably fast. If we find no such marker it means that
	this logfile has been condensed and does not have enough transactions
	to return, then we return 'DO FULL!!'.
	The client will have to ask for the whole file instead.
	If the server has a lower transaction counter than the one sent
	the server is stale, let the client know this.

	This method does not rely on the transactionCounter in the instance."

	| file found result key lastInLog |
	lastInLog _ self lastTransactionInLog.
	lastTransaction = lastInLog ifTrue:[^''].
	lastTransaction > lastInLog ifTrue:[^'STALE SERVER!!'].
	lastTransaction = firstTransactionNumber ifTrue:[^self updatesSinceFirstTransaction ].
	[ file _ self openLogFileReadOnly setToEnd.
	key _ 'self transactionCounter: ', lastTransaction storeString, '!!'.
	found _ file findStringFromEnd: key.
	found = 0 ifTrue:[result _ 'DO FULL!!'] ifFalse: [ file skip: key size. result _ file upToEnd ]]
		ensure: [file close].
	^result! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 11/17/2003 13:06'!
upgradeOldPackages
	"Upgrade all upgradeable old packages without confirmation on each."

	^self upgradeOldPackagesConfirmBlock: [:package | true ]! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 7/14/2004 16:26'!
upgradeOldPackagesConfirmBlock: aBlock
	"First we find out which of the installed packages are upgradeable and old.
	Then we upgrade them if confirmation block yields true.
	The block will be called with each SMPackage to upgrade.
	We return a Dictionary with the packages we tried to upgrade as keys
	and the value being the result of the upgrade, true or false."

	| result |
	result _ Dictionary new.
	self upgradeableAndOldPackages
		do: [:package |
			(aBlock value: package)
				ifTrue:[ result at: package put: package upgrade]].
	^result
! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 7/14/2004 17:33'!
upgradeOrInstallPackage: aPackage
	"Upgrade package (or install) to the latest published release for this Squeak version."

	^aPackage upgradeOrInstall! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 7/14/2004 17:33'!
upgradeOrInstallPackageWithId: anUUIDString
	"Upgrade package (or install) to the latest published release for this Squeak version."

	| package |
	package _ self packageWithId: anUUIDString.
	package ifNil: [self error: 'No package available with id: ''', anUUIDString, ''''].
	^package upgradeOrInstall! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 7/14/2004 17:16'!
upgradePackage: aPackage
	"Upgrade package to the latest published release for this Squeak version."

	^aPackage upgrade! !

!SMSqueakMap methodsFor: 'public-installation' stamp: 'gk 7/14/2004 17:30'!
upgradePackageWithId: anUUIDString
	"Upgrade package to the latest published release for this Squeak version.
	Will raise error if there is no release installed, otherwise use
	#upgradeOrInstallPackageWithId: "

	| package |
	package _ self packageWithId: anUUIDString.
	package ifNil: [self error: 'No package available with id: ''', anUUIDString, ''''].
	^package upgrade! !

!SMSqueakMap methodsFor: 'public-packages' stamp: 'gh 10/25/2002 11:33'!
upgradeableAndOldOrInstallableAndNotInstalledPackages
	"This would give you all packages that are available now
	for automatic install or automatic upgrade."

	^self upgradeableAndOldPackages union: self installableAndNotInstalledPackages! !

!SMSqueakMap methodsFor: 'public-packages' stamp: 'gk 7/14/2004 16:17'!
upgradeableAndOldPackages
	"Answer all packages that are installed and which have a
	newer published release for this Squeak version that also
	can be to by an installer."

	^self installedPackages select: [:package | package isSafelyOldAndUpgradeable]! !

!SMSqueakMap methodsFor: 'public-packages' stamp: 'gh 12/1/2002 19:54'!
upgradeablePackages
	"Answer all packages that has an installer that can upgrade them.
	Note that this doesn't mean they are old, see #upgradeableAndOldPackages."

	^self packages select: [:package | package isUpgradeable]! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 8/3/2003 23:13'!
usernameOccupied: aUsername
	"Return true if name already taken."

	^(self accountForUsername: aUsername) notNil! !

!SMSqueakMap methodsFor: 'accessing' stamp: 'gk 7/30/2003 14:10'!
users
	"Lazily maintain a cache of all known account objects
	keyed by their developer initials."

	users ifNotNil: [^users].
	users _ Dictionary new.
	self accounts do: [:a | users at: a initials put: a].
	^users! !

!SMSqueakMap methodsFor: 'private' stamp: 'gk 10/21/2003 23:05'!
verifyAdminPassword: aString
	"Answer true if it is the correct password."

	^adminPassword = (SecureHashAlgorithm new hashMessage: aString)! !

!SMSqueakMap methodsFor: 'views' stamp: 'gh 11/28/2002 22:06'!
viewFor: uiObject
	"This is a double dispatch mechanism for multiple views
	for multiple uis. Used primarily by the web frontend."

	^uiObject squeakMapViewOn: self! !

Object subclass: #SMUtilities
	instanceVariableNames: ''
	classVariableNames: 'MasterServer MailServer'
	poolDictionaries: ''
	category: 'SMBase-UI'!

!SMUtilities commentStamp: 'gk 11/13/2003 23:39' prior: 0!
Various loose functions in SM.!

!SMUtilities class methodsFor: 'class initialization' stamp: 'gk 1/18/2004 15:12'!
initialize
	"Initialize server settings."

	"self initialize"

	self mailServer: 'mail.bluefish.se' masterServer: 'http://localhost/sm'! !

!SMUtilities class methodsFor: 'private' stamp: 'yo 7/26/2004 22:48'!
mail: anAccount subject: sub message: msg
	"Send a mail to the holder of <anAccount>."

	SMTPClient
		deliverMailFrom: 'squeakmap@squeakfoundation.org'
		to: {anAccount email}
		text:
('From: SqueakMap <squeakmap@squeakfoundation.org>
To: ', anAccount email, '
Subject: ', sub,
'
', msg, (self randomPhrase), ', SqueakMap') usingServer: MailServer! !

!SMUtilities class methodsFor: 'server' stamp: 'gk 8/4/2003 16:15'!
mailPassword: randomPass for: anAccount
	"Change the password to a random generated one
	and mail it to the holder of the account."

	self mail: anAccount subject: 'New password at SqueakMap!!' message:
'Hi!!
An extra random password has been added for the account held by "', anAccount name, '":
"', randomPass, '"

You can login to SqueakMap at:

', MasterServer, '/login

The regular password still works, so if it was not you who requested this extra
random password you can safely just delete this email.

This extra password will stop working when you change your regular password.

'! !

!SMUtilities class methodsFor: 'server' stamp: 'gk 8/4/2003 16:15'!
mailPassword: aPassword forNew: anAccount
	"Mail the password to the person who just registered the account."

	self mail: anAccount subject: 'Your new account at SqueakMap!!' message:
'Hi!!
You or someone else has registered an account on SqueakMap. You can login to it using this link:

',
MasterServer, '/autologin?u=', anAccount initials, '&p=', aPassword,
'

If it was not you who performed this registration you can safely just delete this email.

'! !

!SMUtilities class methodsFor: 'class initialization' stamp: 'gk 1/18/2004 15:11'!
mailServer: ipName masterServer: httpUrl
	"Initialize server settings."

	MailServer _ ipName.
	MasterServer _ httpUrl! !

!SMUtilities class methodsFor: 'private' stamp: 'rbb 3/1/2005 11:12'!
mailUserName
	"Answer the mail user's name, but deal with some historical mail senders."

	| mailSender |
	mailSender := (Smalltalk at: #MailSender ifAbsent: [ Smalltalk at: #Celeste ifAbsent: []]).
	^mailSender
		ifNil: [ UIManager default request: 'What is your email address?' ]
		ifNotNil: [ mailSender userName ]! !

!SMUtilities class methodsFor: 'server' stamp: 'gk 8/15/2003 12:25'!
masterServer
	"Return the master server url."

	^MasterServer! !

!SMUtilities class methodsFor: 'private' stamp: 'gh 11/27/2002 12:37'!
randomPhrase
	"Pick a nice phrase."

	^#('Debug safely' 'Happy Squeaking' 'Just do it' 'Yours truly' 'Stay a Squeaker' 'Squeak rocks') atRandom! !

!SMUtilities class methodsFor: 'utilities' stamp: 'rbb 3/1/2005 11:12'!
sendMail: aString
	"Send the given mail message, but check for modern mail senders."

	| server |

	Smalltalk at: #MailSender ifPresent: [ :mailSender |
		^mailSender sendMessage: ((Smalltalk at: #MailMessage) from: aString).
	].

	Smalltalk at: #MailComposition ifPresent: [ :mailComposition |
		^mailComposition new
			messageText:  aString;
			open
	].
	
	Smalltalk at: #Celeste ifPresent: [ :celeste |
		celeste isSmtpServerSet ifTrue: [
			Smalltalk at: #CelesteComposition ifPresent: [ :celesteComposition |
				^celesteComposition
					openForCeleste: celeste current 
					initialText: aString
			]
		]
	].

	Smalltalk at: #AdHocComposition ifPresent: [ :adHocComposition |
		server :=  UIManager default request: 'What is your mail server for outgoing mail?'.
		^adHocComposition 
			openForCeleste: server
			initialText: aString
	].

	^self inform: 'Sorry, no known way to send the message'.
	 	! !

!SMUtilities class methodsFor: 'utilities' stamp: 'gk 7/10/2004 03:43'!
sendMailTo: recipient regardingPackageRelease: pr
	"Send mail to the given recipient. Try to use the first of:
	- MailSender (with its registered composition class)
	- Celeste
	- AdHocComposition
	for compatibility with 3.5 and 3.6 images"

	self sendMail: (String streamContents: [:stream |
		stream
			nextPutAll: 'From: '; nextPutAll: self mailUserName; cr;
			nextPutAll: 'To: '; nextPutAll: recipient; cr;
			nextPutAll: 'Subject: Regarding '; nextPutAll: pr printName; cr])! !

!SMUtilities class methodsFor: 'utilities' stamp: 'gk 7/9/2004 20:08'!
stripEmailFrom: aString
	"Picks out the email from:
		'Robert Robertson <rob@here.com>' => 'rob@here.com'
	Spamblockers 'no_spam', 'no_canned_ham' and 'spam_block'
	(case insensitive) will be filtered out."

	| lessThan moreThan email pos |
	lessThan _ aString indexOf: $<.
	moreThan _ aString indexOf: $>.
	(lessThan * moreThan = 0) ifTrue: [^ aString].
	email _ (aString copyFrom: lessThan + 1 to: moreThan - 1) asLowercase.
	#('no_spam' 'no_canned_ham' 'spam_block') do: [:block |
		pos _ email findString: block.
		pos = 0 ifFalse:[email _ (email copyFrom: 1 to: pos - 1), (email copyFrom: pos + block size to: email size)]].
	^email! !

!SMUtilities class methodsFor: 'utilities' stamp: 'gk 8/4/2003 11:55'!
stripNameFrom: aString
	"Picks out the name from:
		'Robert Robertson <rob@here.com>' => 'Robert Robertson'
	"

	| lessThan |
	lessThan _ aString indexOf: $<.
	^(aString copyFrom: 1 to: lessThan - 1) withBlanksTrimmed ! !
SMSqueakMap initialize!
SMUtilities initialize!
